#include <sstream>
#include <stdexcept>
#include "Matrix.hpp"

using std::abs;
using std::ostringstream;
using std::invalid_argument;

namespace mathkit {
	
	Matrix::Matrix() : _data(), _dimen(0, 0) {}
	
	Matrix::Matrix(const Table & data) {
		_data = data;
		_dimen = Dimen(data[0].size(), data.size());
	}
	
	Matrix::Matrix(Dimen dimen, double val) {
		_data = Table(dimen.second, Vector(dimen.first, val));
		_dimen = dimen;
	}
	
	Matrix::Matrix(const Vector & data, Dimen dimen, bool byrow) {
		setData(data, dimen, byrow);
	}
	
	void Matrix::setData(const Table & data) {
		_data = data;
		_dimen = Dimen(data[0].size(), data.size());
	}
	
	void Matrix::setData(Dimen dimen, double val) {
		_data = Table(dimen.second, Vector(dimen.first, val));
		_dimen = dimen;
	}
	
	void Matrix::setData(const Vector & data, Dimen dimen, bool byrow) {
		_dimen = dimen;
		_data = Table(dimen.second);
		if (byrow) {
			int k = 0;
			for (int i = 0; i < dimen.first; ++i)
				for (int j = 0; j < dimen.second; ++j)
					_data[j].push_back(data[k++]);
		}
		else {
			int k = 0;
			for (int i = 0; i < dimen.second; ++i)
				for (int j = 0; j < dimen.first; ++j)
					_data[i].push_back(data[k++]);
		}
	}
	
	Table Matrix::getData() const {
		return _data;
	}
	
	Dimen Matrix::dimen() const {
		return _dimen;
	}
	
	bool Matrix::dimen(Dimen dimen) {
		if (!_data.empty()) return false;
		_dimen = dimen;
		return true;
	}
	
	int Matrix::nrow() const {
		return _dimen.first;
	}
	
	int Matrix::ncol() const {
		return _dimen.second;
	}
	
	Matrix Matrix::round(int ndec) const {
		Matrix mat = *this;
		int m = mat._dimen.first, n = mat._dimen.second;
		for (int i = 0; i < m; ++i)
			for (int j = 0; j < n; ++j)
				mat(i, j) = prec(mat(i, j), ndec);
		return mat;
	}
	
	bool Matrix::square() const {
		return _dimen.first == _dimen.second;
	}
	
	bool Matrix::symmetric() const {
		if (!square()) return false;
		return *this == transpose();
	}
	
	Vector Matrix::getRow(int m) const {
		Vector row(_dimen.second);
		for (int i = 0; i < _dimen.second; ++i) row[i] = _data[i][m];
		return row;
	}
	
	Vector Matrix::getCol(int n) const {
		return _data[n];
	}
	
	void Matrix::setRow(int m, const Vector & row) {
		for (int i = 0; i < _dimen.second; ++i) _data[i][m] = row[i];
	}
	
	void Matrix::setCol(int n, const Vector & col) {
		_data[n] = col;
	}
	
	void Matrix::exchange(int i, int j, bool row) {
		Vector temp;
		if (row) {
			temp = getRow(i);
			setRow(i, getRow(j));
			setRow(j, temp);
		}
		else {
			temp = getCol(i);
			setCol(i, getCol(j));
			setCol(j, temp);
		}
	}
	
	Matrix Matrix::transpose() const {
		Table tbl;
		for (int i = 0; i < _dimen.first; ++i) tbl.push_back(getRow(i));
		return Matrix(tbl);
	}
	
	int Matrix::rank(const Epsilon & eps) const {
		Matrix mat = *this;
		int m = mat._dimen.first;
		int n = mat._dimen.second;
		int limit = m < n ? m : n;
		int r = 0;
		for (int i = 0; i < n; ++i) {
			if (eps(mat(r, i))) {
				bool exchanged = false;
				for (int j = r + 1; j < m; ++j) {
					if (!eps(mat(j, i))) {
						mat.exchange(r, j);
						exchanged = true;
						break;
					}
				}
				if (!exchanged) continue;
			}
			for (int k = r + 1; k < m; ++k) {
				if (!eps(mat(k, i))) {
					double s = mat(k, i) / mat(r, i);
					mat.setRow(k, -s * mat.getRow(r) + mat.getRow(k));
				}
			}
			r += 1;
			if (r == limit) break;
		}
		return r;
	}
	
	Matrix Matrix::submat(int frow, int lrow, int fcol, int lcol) {
		int m = lrow - frow + 1, n = lcol - fcol + 1;
		Table tbl(n, Vector(m));
		for (int i = 0; i < m; ++i)
			for (int j = 0; j < n; ++j)
				tbl[j][i] = _data[fcol + j][frow + i];
		return Matrix(tbl);
	}
	
	Matrix Matrix::resid(int i, int j) const {
		Table tbl;
		for (int n = 0; n < _dimen.second; ++n) {
			if (n != j) {
				Vector col = _data[n];
				Vector::iterator iter = col.begin();
				for (int m = 0; m < i; ++m) ++iter;
				col.erase(iter);
				tbl.push_back(col);
			}
		}
		return Matrix(tbl);
	}
	
	double & Matrix::operator()(int m, int n) {
		return _data[n][m];
	}
	
	double Matrix::operator()(int m, int n) const {
		return _data[n][m];
	}
	
	Matrix Matrix::operator+(const Matrix & mat) const {
		if (_dimen != mat._dimen) throw invalid_argument("Incompatible matrices.");
		Table tbl;
		for (int i = 0; i < _dimen.second; ++i) tbl.push_back(_data[i] + mat._data[i]);
		return Matrix(tbl);
	}
	
	Matrix Matrix::operator-(const Matrix & mat) const {
		if (_dimen != mat._dimen) throw invalid_argument("Incompatible matrices.");
		Table tbl;
		for (int i = 0; i < _dimen.second; ++i) tbl.push_back(_data[i] - mat._data[i]);
		return Matrix(tbl);
	}
	
	Matrix Matrix::operator*(const Matrix & mat) const {
		if (_dimen.second != mat._dimen.first) throw invalid_argument("Incompatible matrices.");
		Table tbl(mat._dimen.second, Vector(_dimen.first, 0));
		for (int i = 0; i < mat._dimen.second; ++i)
			for (int j = 0; j < _dimen.first; ++j)
				tbl[i] = tbl[i] + mat._data[i][j] * _data[j];
		return Matrix(tbl);
	}
	
	bool Matrix::operator==(const Matrix & mat) const {
		return _dimen == mat._dimen && _data == mat._data;
	}
	
	bool Matrix::operator!=(const Matrix & mat) const {
		return !(_dimen == mat._dimen && _data == mat._data);
	}
	
	Matrix operator*(const Matrix & mat, double scalar) {
		Table tbl;
		int n = mat.ncol();
		for (int i = 0; i < n; ++i) tbl.push_back(scalar * mat.getCol(i));
		return Matrix(tbl);
	}
	
	Matrix operator*(double scalar, const Matrix & mat) {
		return mat * scalar;
	}
	
	ostream & operator<<(ostream & outs, const Matrix & mat) {
		save(outs, mat.getData());
		return outs;
	}
	
	istream & operator>>(istream & ins, Matrix & mat) {
		if (mat.dimen() != Dimen(0, 0))
			mat.setData(load(ins, mat.nrow(), mat.ncol()));
		return ins;
	}
	
	Matrix t(const Matrix & mat) {
		return mat.transpose();
	}
	
	Matrix eye(int n) {
		Matrix mat(Dimen(n, n));
		for (int i = 0; i < n; ++i) mat(i, i) = 1;
		return mat;
	}
	
	Vector diag(const Matrix & mat) {
		Vector vec;
		int n = mat.dimen().second;
		for (int i = 0; i < n; ++i) vec.push_back(mat(i, i));
		return vec;
	}
	
	Matrix diag(const Vector & vec) {
		int n = vec.size();
		Matrix mat(Dimen(n, n));
		for (int i = 0; i < n; ++i) mat(i, i) = vec[i];
		return mat;
	}
	
	double det(const Matrix & mat, const Epsilon & eps) {
		if (!mat.square()) throw invalid_argument("Square matrix needed.");
		int n = mat.ncol();
		if (n == 1) return eps(mat(0, 0)) ? 0 : mat(0, 0);
		double result = 0;
		for (int i = 0; i < n; ++i) {
			if (!eps(mat(0, i))) {
				double resid = mat(0, i) * det(mat.resid(0, i));
				if (i % 2 != 0) resid = -resid;
				result += resid;
			}
		}
		return result;
	}
	
	Matrix inv(const Matrix & mat, const Epsilon & eps) {
		if (!mat.square()) throw invalid_argument("Square matrix needed.");
		int n = mat.ncol();
		Matrix I = eye(n);
		Table tbl;
		Matrices lup = lu(mat, eps);
		for (int i = 0; i < n; ++i) {
			Vector result = lusolve(lup, I.getCol(i), eps);
			if (result.empty()) throw invalid_argument("Singular matrix.");
			tbl.push_back(result);
		}
		return Matrix(tbl);
	}
	
	double trace(const Matrix & mat, const Epsilon & eps) {
		if (!mat.square()) throw invalid_argument("Square matrix needed.");
		int n = mat.ncol();
		double val = 0;
		for (int i = 0; i < n; ++i)
			if (!eps(mat(i, i))) val += mat(i, i);
		return val;
	}
	
	bool positive(const Matrix & mat, const Epsilon & eps) {
		if (!mat.symmetric()) throw invalid_argument("Symmetric matrix needed.");
		Matrices lup = lu(mat, eps);
		if (lup.empty()) return false;
		const Matrix &U = lup[1];
		int n = U.ncol();
		for (int i = 0; i < n; ++i)
			if (U(i, i) < 0) return false;
		return true;
	}
	
	Table table(const Vector & data, Dimen dimen, bool byrow) {
		Table tbl(dimen.second);
		if (byrow) {
			int k = 0;
			for (int i = 0; i < dimen.first; ++i)
				for (int j = 0; j < dimen.second; ++j)
					tbl[j].push_back(data[k++]);
		}
		else {
			int k = 0;
			for (int i = 0; i < dimen.second; ++i)
				for (int j = 0; j < dimen.first; ++j)
					tbl[i].push_back(data[k++]);
		}
		return tbl;
	}
	
	Matrix rowmat(const Vector & data) {
		return Matrix(data, Dimen(1, data.size()));
	}
	
	Matrix colmat(const Vector & data) {
		return Matrix(data, Dimen(data.size(), 1));
	}
	
	Matrix cbind(const Matrix & mat1, const Matrix & mat2) {
		if (mat1.nrow() != mat2.nrow()) throw invalid_argument("Incompatible matrices.");
		Table tbl;
		for (int i = 0; i < mat1.ncol(); ++i) tbl.push_back(mat1.getCol(i));
		for (int i = 0; i < mat2.ncol(); ++i) tbl.push_back(mat2.getCol(i));
		return Matrix(tbl);
	}
	
	Matrix rbind(const Matrix & mat1, const Matrix & mat2) {
		int n = mat1.ncol();
		if (n != mat2.ncol()) throw invalid_argument("Incompatible matrices.");
		Table tbl(n);
		for (int i = 0; i < mat1.nrow(); ++i) {
			Vector row = mat1.getRow(i);
			for (int j = 0; j < n; ++j) tbl[j].push_back(row[j]);
		}
		for (int i = 0; i < mat2.nrow(); ++i) {
			Vector row = mat2.getRow(i);
			for (int j = 0; j < n; ++j) tbl[j].push_back(row[j]);
		}
		return Matrix(tbl);
	}
	
	Matrices lu(const Matrix & mat, const Epsilon & eps) {
		if (!mat.square()) throw invalid_argument("Square matrix needed.");
		int n = mat.ncol();
		Matrix L = eye(n), P = eye(n);
		Matrix U = mat;
		for (int i = 0; i < n; ++i) {
			if (eps(U(i, i))) {
				bool exchanged = false;
				for (int k = i + 1; k < n; ++k) {
					if (!eps(U(k, i))) {
						U.exchange(i, k);
						P.exchange(i, k);
						for (int j = 0; j < i; ++j) {
							double temp = L(i, j);
							L(i, j) = L(k, j);
							L(k, j) = temp;
						}
						exchanged = true;
						break;
					}
				}
				if (!exchanged) return Matrices();
			}
			for (int j = i + 1; j < n; ++j) {
				if (!eps(U(j, i))) {
					double s = U(j, i) / U(i, i);
					L(j, i) = s;
					U.setRow(j, -s * U.getRow(i) + U.getRow(j));
				}
			}
		}
		Matrices result;
		result.push_back(L);
		result.push_back(U);
		result.push_back(P);
		return result;
	}
	
	Matrices qr(const Matrix & mat, const Epsilon & eps) {
		int n = mat.ncol();
		if (mat.rank(eps) < n) return Matrices();
		Table tbl;
		for (int i = 0; i < n; ++i) {
			Vector vec = mat.getCol(i);
			for (int j = 0; j < i; ++j) vec = vec - tbl[j] * dot_prod(tbl[j], vec);
			tbl.push_back(vec / norm(vec));
		}
		Matrix Q(tbl);
		Matrix R(Dimen(n, n));
		for (int i = 0; i < n; ++i)
			for (int j = i; j < n; ++j)
				R(i, j) = dot_prod(tbl[i], mat.getCol(j));
		Matrices result;
		result.push_back(Q);
		result.push_back(R);
		return result;
	}
	
	Vector linsolve(const Matrix & mat, const Vector & vec, const Epsilon & eps) {
		return lusolve(lu(mat, eps), vec, eps);
	}
	
	Vector lusolve(const Matrices & lup, const Vector & vec, const Epsilon & eps) {
		if (lup.empty()) return Vector();
		const Matrix &L = lup[0], &U = lup[1], &P = lup[2];
		unsigned int n = U.nrow();
		if (n != vec.size()) throw invalid_argument("Incompatible arguments.");
		Matrix b = colmat(vec);
		if (P != eye(n)) b = P * b;
		Vector result;
		for (unsigned int i = 0; i < n; ++i) {
			double val = 0;
			for (unsigned int j = 0; j < i; ++j)
				if (!eps(L(i, j)))val += L(i, j) * result[j];
			result.push_back(b(i, 0) - val);
		}
		b = colmat(result);
		for (int i = n - 1; i >= 0; --i) {
			double val = 0;
			for (int j = n - 1; j > i; --j)
				if (!eps(U(i, j))) val += U(i, j) * result[j];
			result[i] = (b(i, 0) - val) / U(i, i);
		}
		return result;
	}
	
	Vector qrsolve(const Matrices & qrmat, const Vector & vec, const Epsilon & eps) {
		if (qrmat.empty()) return Vector();
		const Matrix &Q = qrmat[0], &R = qrmat[1];
		unsigned int m = Q.nrow();
		if (vec.size() != m) throw invalid_argument("Incompatible arguments.");
		Matrix b = colmat(vec);
		b = t(Q) * b;
		int n = R.nrow();
		Vector result(n);
		for (int i = n - 1; i >= 0; --i) {
			double val = 0;
			for (int j = n - 1; j > i; --j)
				if (!eps(R(i, j))) val += R(i, j) * result[j];
			result[i] = (b(i, 0) - val) / R(i, i);
		}
		return result;
	}
	
	string to_str(const Matrix & mat, bool byrow, string delim) {
		ostringstream oss;
		if (byrow) {
			int m = mat.nrow();
			for (int i = 0; i < m; ++i) oss << to_str(mat.getRow(i));
		}
		else {
			int n = mat.ncol();
			for (int i = 0; i < n; ++i) oss << to_str(mat.getCol(i));
		}
		string result = oss.str();
		if (delim.size() == 2) result = delim[0] + result + delim[1];
		return result;
	}
}
