#ifndef MATRIX_HPP
#define MATRIX_HPP

#include "mathkit.hpp"

namespace mathkit {
	/**
	 * Dimen is just a convenient alias for pair<int, int>.
	 */
	typedef pair<int, int> Dimen;
	
	/**
	 * Matrix class. The row and column indices start from zero.
	 */
	class Matrix {
	private:
		Table _data;
		Dimen _dimen;
	public:
		/**
		 * Default constructor.
		 */
		Matrix();
		
		/**
		 * Constructor with initial data.
		 * @param data The initial data.
		 */
		Matrix(const Table & data);
		
		/**
		 * Constructor with a specific initial value.
		 * @param dimen The dimension of the matrix.
		 * @param val The specific initial value.
		 */
		Matrix(Dimen dimen, double val = 0);
		
		/**
		 * Constructor with initial data.
		 * @param data A vector contains the initial data.
		 * @param dimen The dimension of the matrix.
		 * @param byrow The initial data is in row major or column major form.
		 */
		Matrix(const Vector & data, Dimen dimen, bool byrow = true);
		
		/**
		 * Set matrix data.
		 * @param data The new matrix data.
		 */
		void setData(const Table & data);
		
		/**
		 * Set Matrix data with a specific value.
		 * @param dimen The dimension of the matrix.
		 * @param val The specific value.
		 */
		void setData(Dimen dimen, double val = 0);
		
		/**
		 * Set matrix data.
		 * @param data A vector contains the new data.
		 * @param dimen The dimension of the matrix.
		 * @param byrow The new data is in row major or column major form.
		 */
		void setData(const Vector & data, Dimen dimen, bool byrow = true);
		
		/**
		 * Get matrix data.
		 * @return The matrix data.
		 */
		Table getData() const;
		
		/**
		 * Get matrix dimension.
		 * @return The dimension of the matrix.
		 */
		Dimen dimen() const;
		
		/**
		 * Set matrix dimension.
		 * The dimension could be set only when the matrix data is empty. Else this operation would fail.@n
		 * This method should only be used when the matrix is contructed by default contructor and need to load data using stream extraction operator (>>).
		 * @param dimen The dimension.
		 * @return Whether the operation succeed.
		 */
		bool dimen(Dimen dimen);
		
		/**
		 * The number of rows.
		 * @return The number of rows.
		 */
		int nrow() const;
		
		/**
		 * The number of columns.
		 * @return The number of columns.
		 */
		int ncol() const;
		
		/**
		 * Round every element to specific decimal digits.
		 * @param ndec The number of decimal digits.
		 * @return A matrix with each element rounded to specific decimal digits.
		 */
		Matrix round(int ndec = 4) const;
		
		/**
		 * Test if the matrix is a square matrix.
		 * @return True if the matrix is square else false.
		 */
		bool square() const;
		
		/**
		 * Test if the matrix is symmetric.
		 * @return True if the matrix is symmetric else false.
		 */
		bool symmetric() const;
		
		/**
		 * Get a row data.
		 * @param m The row index (start from zero).
		 * @return A vector contains the specific row data.
		 */
		Vector getRow(int m) const;
		
		/**
		 * Get a column data.
		 * @param n The column index (start from zero).
		 * @return A vector contains the specific column data.
		 */
		Vector getCol(int n) const;
		
		/**
		 * Set a row data.
		 * @param m The row index (start from zero).
		 * @param row The new row data.
		 */
		void setRow(int m, const Vector & row);
		
		/**
		 * Set a column data.
		 * @param n The column index (start from zero).
		 * @param col The new column data.
		 */
		void setCol(int n, const Vector & col);
		
		/**
		 * Exchange two rows or two columns.
		 * @param i The index of first row/column to exchange.
		 * @param j The index of second row/column to exchange.
		 * @param row Which stuff to exchange (row or column).
		 */
		void exchange(int i, int j, bool row = true);
		
		/**
		 * Transpose matrix.
		 * @return The transposed matrix.
		 */
		Matrix transpose() const;
		
		/**
		 * Rank of matrix.
		 * @param eps An Epsilon functor to test zero elements.
		 * @return The rank of the matrix.
		 */
		int rank(const Epsilon & eps = Epsilon()) const;
		
		/**
		 * Submatrix.
		 * @param frow The first row index.
		 * @param lrow The last row index.
		 * @param fcol The first column index.
		 * @param lcol The last column index.
		 * @return The submatrix.
		 */
		Matrix submat(int frow, int lrow, int fcol, int lcol);
		
		/**
		 * Residual matrix.
		 * @param i The row index (zero based).
		 * @param j The column index (zero based).
		 * @return A matrix with the ith row and jth column excluded.
		 */
		Matrix resid(int i, int j) const;
		
		/**
		 * Access a specific element of the matrix.
		 * @param m The row index (start from zero).
		 * @param n The column index (start from zero).
		 * @return A reference of the specific element.
		 */
		double & operator()(int m, int n);
		
		/**
		 * Access a specific element of the matrix (read only).
		 * @param m The row index (start from zero).
		 * @param n The column index (start from zero).
		 * @return The specific element.
		 */
		double operator()(int m, int n) const;
		
		/**
		 * Matrix addition.
		 * @param mat The right matrix operand.
		 * @return A matrix contains the result of addition.
		 */
		Matrix operator+(const Matrix & mat) const;
		
		/**
		 * Matrix subtraction.
		 * @param mat The right matrix operand.
		 * @return A matrix contains the result of subtraction.
		 */
		Matrix operator-(const Matrix & mat) const;
		
		/**
		 * Matrix multiplication.
		 * @param mat The right matrix operand.
		 * @return A matrix contains the result of multiplication.
		 */
		Matrix operator*(const Matrix & mat) const;
		
		/**
		 * Matrix equality.
		 * @param mat The right matrix operand.
		 * @return Whether the matrices are equal.
		 */
		bool operator==(const Matrix & mat) const;
		
		/**
		 * Matrix inequality.
		 * @param mat The right matrix operand.
		 * @return Whether the matrices are inequal.
		 */
		bool operator!=(const Matrix & mat) const;
	};
	
	/**
	 * Matrices is just a convenient alias for vector<Matrix>.
	 */
	typedef vector<Matrix> Matrices;
	
	/**
	 * Scalar multiplication for matrix.
	 * @param mat The matrix.
	 * @param scalar The scalar value.
	 * @return A matrix contains the result.
	 */
	Matrix operator*(const Matrix & mat, double scalar);
	
	/**
	 * Scalar multiplication for matrix.
	 * @param scalar The scalar value.
	 * @param mat The matrix.
	 * @return A matrix contains the result.
	 */
	Matrix operator*(double scalar, const Matrix & mat);
	
	/**
	 * Stream insertion operator for matrix.
	 * @param outs The output stream.
	 * @param mat The matrix.
	 * @return The reference of the output stream.
	 */
	ostream & operator<<(ostream & outs, const Matrix & mat);
	
	/**
	 * Stream extraction operator for matrix.
	 * @param ins The input stream.
	 * @param mat The matrix.
	 * @return The reference of the input stream.
	 */
	istream & operator>>(istream & ins, Matrix & mat);
	
	/**
	 * Transpose matrix.
	 * @param mat The original matrix.
	 * @return The transposed matrix.
	 */
	Matrix t(const Matrix & mat);
	
	/**
	 * Generate an identity matrix (n by n).
	 * @param n The number of rows/columns of the matrix.
	 * @return The identity matrix.
	 */
	Matrix eye(int n);
	
	/**
	 * Get the diagonal elements of a square matrix.
	 * @param mat The square matrix.
	 * @return A vector contains the diagonal elements.
	 */
	Vector diag(const Matrix & mat);
	
	/**
	 * Generate a diagonal matrix with specific diagonal elements.
	 * @param vec A vector contains the diagonal elements.
	 * @return The diagonal matrix.
	 */
	Matrix diag(const Vector & vec);
	
	/**
	 * Determinant.
	 * @param mat The square matrix to calculate determinant.
	 * @param eps An Epsilon functor to test zero elements.
	 * @return The determinant of the square matrix.
	 */
	double det(const Matrix & mat, const Epsilon & eps = Epsilon());
	
	/**
	 * Inverse matrix.
	 * @param mat The original matrix.
	 * @param eps An Epsilon functor to test zero elements.
	 * @return The inverse matrix.
	 */
	Matrix inv(const Matrix & mat, const Epsilon & eps = Epsilon());
	
	/**
	 * Trace (sum of main diagonal elements) of square matrix.
	 * @param mat The square matrix.
	 * @param eps An Epsilon functor to test zero elements.
	 * @return The trace of the square matrix.
	 */
	double trace(const Matrix & mat, const Epsilon & eps = Epsilon());
	
	/**
	 * Check whether a symmetric matrix is positive definite.
	 * @param mat The symmetric matrix.
	 * @param eps An Epsilon functor to test zero elements.
	 * @return True if the matrix is positive definite else false.
	 */
	bool positive(const Matrix & mat, const Epsilon & eps = Epsilon());
	
	/**
	 * Generate a table from a vector.
	 * @param data A vector contains the table data.
	 * @param dimen The dimension of the table.
	 * @param byrow The data is in row major or column major form.
	 * @return A table.
	 */
	Table table(const Vector & data, Dimen dimen, bool byrow = true);
	
	/**
	 * Generate an one row matrix (row vector in linear algebra).
	 * @param data A vector contains the data.
	 * @return An one row matrix.
	 */
	Matrix rowmat(const Vector & data);
	
	/**
	 * Generate an one column matrix (column vector in linear algebra).
	 * @param data A vector contains the data.
	 * @return An one column matrix.
	 */
	Matrix colmat(const Vector & data);
	
	/**
	 * Construct a matrix by column binding.
	 * @param mat1 The first matrix.
	 * @param mat2 The second matrix.
	 * @return A matrix binds the columns of mat1 and mat2.
	 */
	Matrix cbind(const Matrix & mat1, const Matrix & mat2);
	
	/**
	 * Construct a matrix by row binding.
	 * @param mat1 The first matrix.
	 * @param mat2 The second matrix.
	 * @return A matrix binds the rows of mat1 and mat2.
	 */
	Matrix rbind(const Matrix & mat1, const Matrix & mat2);
	
	/**
	 * LU decomposition.
	 * @param mat The matrix to decomposite.
	 * @param eps An Epsilon functor to test zero elements.
	 * @return A vector contains the L (lower triangular), U (upper triangular) and P (permutation) matrices if succeed, else an empty vector.
	 */
	Matrices lu(const Matrix & mat, const Epsilon & eps = Epsilon());
	
	/**
	 * QR decomposition.
	 * @param mat The matrix to decomposite.
	 * @param eps An Epsilon functor to test zero elements.
	 * @return If succeed return a vector contains the matrices Q and R, else return an empty vector.
	 */
	Matrices qr(const Matrix & mat, const Epsilon & eps = Epsilon());
	
	/**
	 * Solve a linear system (Ax = b).
	 * @param mat The coefficients matrix of the linear system.
	 * @param vec The column vector on the right side.
	 * @param eps An Epsilon functor to test zero elements.
	 * @return If succeed return a vector contains the solutions else return an empty vector.
	 */
	Vector linsolve(const Matrix & mat, const Vector & vec, const Epsilon & eps = Epsilon());
	
	/**
	 * Solve a linear system represented by L, U, P (Ax = b, PA = LU).
	 * @param lup A vector contains the L, U and P matrices.
	 * @param vec The column vector on the right side.
	 * @param eps An Epsilon functor to test zero elements.
	 * @return A vector contains the solutions if succeed, else an empty vector.
	 */
	Vector lusolve(const Matrices & lup, const Vector & vec, const Epsilon & eps = Epsilon());
	
	/**
	 * Solve a linear system represented by Q, R (Ax = b, A = QR).
	 * @param qrmat A vector contains the Q and R matrices.
	 * @param vec The column vector on the right side.
	 * @param eps An Epsilon functor to test zero elements.
	 * @return If succeed return a vector contains the solutions, else return an empty vector.
	 */
	Vector qrsolve(const Matrices & qrmat, const Vector & vec, const Epsilon & eps = Epsilon());
	
	/**
	 * The string representation of a matrix.
	 * @param mat The matrix.
	 * @param byrow Whether the string representation is in row major or column major form.
	 * @param delim The delimiter string.
	 * @return The string representation of the matrix.
	 */
	string to_str(const Matrix & mat, bool byrow = true, string delim = "[]");
}

#endif
