/**
 * @mainpage
 * @section intro Introduction
 *
 * Mathkit is a lightweight C++ library for finance, statistics and linear algebra.@n
 * Finance features include compound rate present/future value, annuity, various present/future value coefficients ...@n
 * Statistics features include mean, median, variance, standard deviation, covariance, correlation, linear regression, probabilities and random variates of various distributions ...@n
 * Linear algebra features include matrix arithmetic, inverse, determinant, rank, linear system solution, lu/qr decomposition ...@n
 * And there are also some auxiliary features like random number generators, equation solution, numerical integration, permutation/combination and gcd/lcm etc.
 *
 * It is written in standard C++ with STL and uses STL containers (especially vector) to hold data.
 * So this project is not for you if you have millions and millions of complex data and need extremely high computing efficiency.
 * It is aimed to be a lightweight, convenient and ready-made toolkit for you to check out your ideals.
 *
 * @section inst Installation
 * 
 * Mathkit is written in standard C++ with STL. So you can use any standard C++ compiler to compile it.@n
 * There are also some scripts to help you build it. For linux and other unix-like systems with gnu compilers try Makefile.
 * For windows systems above xp/2000 try build_vc.bat (visual c++ compiler) or build_bc.bat (borland c++ compiler).
 * Then just copy the static library generated and the include directory to anywhere you want to use it later.
 */

#ifndef MATHKIT_HPP
#define MATHKIT_HPP

#include <cmath>
#include <vector>
#include <string>
#include <utility>
#include <iostream>

using std::abs;
using std::atan;
using std::vector;
using std::string;
using std::pair;
using std::make_pair;
using std::istream;
using std::ostream;

/**
 * All functions of mathkit are in the mathkit namespace.
 */
namespace mathkit {
	
	/**
	 * Constant pi.
	 */
	const double pi = 4 * atan(1.0);
	
	/**
	 * Vector is just a convenient alias for vector<double>.
	 */
	typedef vector<double> Vector;
	
	/**
	 * Table is just a convenient alias for vector<vector<double> >.
	 */
	typedef vector<vector<double> > Table;
	
	/**
	 * Pair is just a convenient alias for pair<double, double>.
	 */
	typedef pair<double, double> Pair;
	
	/**
	 * Pairs is just a convenient alias for vector<pair<double, double> >.
	 */
	typedef vector<pair<double, double> > Pairs;
	
	/**
	 * Epsilon functor (test whether |x| < eps).
	 */
	class Epsilon {
	private:
		double _eps;
	public:
		/**
		 * Construct an Epsilon object.
		 * @param eps The threshold value.
		 */
		Epsilon(double eps = 1e-6) {_eps = eps;}
		
		/**
		 * Test whether |val| < eps.
		 * @param val The value to test.
		 * @return True if |val| < eps else false.
		 */
		bool operator()(double val) const;
	};
	
	/**
	 * Parameters (slope and intercept) of regression line.
	 */
	struct LineParam {
		/**
		 * The slope of regression line.
		 */
		double slope;
		
		/**
		 * The intercept of regression line on y axis.
		 */
		double intercept;
	};
	
	/**
	 * Present value of compound interest.
	 * @param amount The terminal/final value.
	 * @param rate The compound interest rate.
	 * @param period The number of periods spanned.
	 * @return The present value.
	 */
	double pv(double amount, double rate, double period);
	
	/**
	 * Future value of compound interest.
	 * @param amount The present value (principal).
	 * @param rate The compound interest rate.
	 * @param period The number of periods spanned.
	 * @return The future/final value.
	 */
	double fv(double amount, double rate, double period);
	
	/**
	 * Present value coefficient of compound interest.
	 * @param rate The compound interest rate.
	 * @param period The number of periods spanned.
	 * @return The present value coefficient.
	 */
	double pv_coef(double rate, double period);
	
	/**
	 * Future value coefficient of compound interest.
	 * @param rate The compound interest rate.
	 * @param period The number of periods spanned.
	 * @return The future value coefficient.
	 */
	double fv_coef(double rate, double period);
	
	/**
	 * Present value of annuity.
	 * @param annuity The payment of every period.
	 * @param rate The compound interest (discount) rate.
	 * @param period The number of periods spanned.
	 * @param prepaid Whether the annuity is an ordinary annuity or a prepaid annuity.
	 * @return The present value of annuity.
	 */
	double apv(double annuity, double rate, int period, bool prepaid = false);
	
	/**
	 * Future value of annuity.
	 * @param annuity The payment of every period.
	 * @param rate The compound interest rate.
	 * @param period The number of periods spanned.
	 * @param prepaid Whether the annuity is an ordinary annuity or a prepaid annuity.
	 * @return The future/final value of annuity.
	 */
	double afv(double annuity, double rate, int period, bool prepaid = false);
	
	/**
	 * Present value coefficient of annuity.
	 * @param rate The compound interest (discount) rate.
	 * @param period The number of periods spanned.
	 * @param prepaid Whether the annuity is an ordinary annuity or a prepaid annuity.
	 * @return The present value coefficient of annuity.
	 */
	double apv_coef(double rate, int period, bool prepaid = false);
	
	/**
	 * Future value coefficient of annuity.
	 * @param rate The compound interest rate.
	 * @param period The number of periods spanned.
	 * @param prepaid Whether the annuity is an ordinary annuity or a prepaid annuity.
	 * @return The future value coefficient of annuity.
	 */
	double afv_coef(double rate, int period, bool prepaid = false);
	
	/**
	 * Present value of a series of various payments.
	 * This function can also be used to calculate net present value (NPV).
	 * @param amount The payment of every period.
	 * @param rate The compound interest (discount) rate.
	 * @param prepaid Whether these payments are ordinary or prepaid (paid at the beginning of every period).
	 * @return The present value of the payments.
	 */
	double spv(const Vector & amount, double rate, bool prepaid = false);
	
	/**
	 * Future value of a series of various payments.
	 * @param amount The payment of every period.
	 * @param rate The compound interest rate.
	 * @param prepaid Whether these payments are ordinary or prepaid (paid at the beginning of every period).
	 * @return The future/final value of the payments.
	 */
	double sfv(const Vector & amount, double rate, bool prepaid = false);
	
	/**
	 * Compound interest/growth rate.
	 * This function can also be used to calculate geometric mean because their formulas are the same.
	 * @param pval The present value (principal).
	 * @param fval The future/final value.
	 * @param period The number of periods spanned.
	 * @return The compound interest/growth rate.
	 */
	double comp_rate(double pval, double fval, double period);
	
	/**
	 * Arithmatic mean.
	 * @param data The population or sample data.
	 * @return The arithmatic mean.
	 */
	double mean(const Vector & data);
	
	/**
	 * Median of a sequence of data.
	 * @param data The population or sample data.
	 * @param sorted Whether the data are sorted.
	 * @return The median value.
	 */
	double median(const Vector & data, bool sorted = false);
	
	/**
	 * Variance of the population or sample data.
	 * For population the variance is normalized by n. And for sample data it is normalized by n - 1.
	 * @param data The population or sample data.
	 * @param sample Whether the data are population data or sample data.
	 * @return The variance value.
	 */
	double var(const Vector & data, bool sample = true);
	
	/**
	 * Standard deviation.
	 * For population the standard deviation is normalized by n. And for sample data it is normalized by n - 1.
	 * @param data The population or sample data.
	 * @param sample Whether the data are population data or sample data.
	 * @return The standard deviation value.
	 */
	double sd(const Vector & data, bool sample = true);
	
	/**
	 * Covariance of two groups of data.
	 * For population the covariance is normalized by n. And for sample data it is normalized by n - 1.
	 * @param data1 The population or sample data of one group.
	 * @param data2 The population or sample data of another group.
	 * @param sample Whether the data are population data or sample data.
	 * @return The covariance value.
	 */
	double cov(const Vector & data1, const Vector & data2, bool sample = true);
	
	/**
	 * Correlation coefficient.
	 * @param data1 The data of one group.
	 * @param data2 The data of another group.
	 * @return The correlation coefficient.
	 */
	double cor(const Vector & data1, const Vector & data2);
	
	/**
	 * Moment.
	 * @param data A vector contains the data.
	 * @param k The degree of the moment.
	 * @param central Central moment if true, else original moment.
	 * @return The moment.
	 */
	double moment(const Vector & data, int k, bool central = true);
	
	/**
	 * Skewness.
	 * @param data A vector contains the data.
	 * @return The skewness.
	 */
	double skew(const Vector & data);
	
	/**
	 * Kurtosis.
	 * @param data A vector contains the data.
	 * @param excess Whether to compute excess kurtosis.
	 * @return The kurtosis.
	 */
	double kurt(const Vector & data, bool excess = true);
	
	/**
	 * Probability density of normal distribution.
	 * @param x A specific value of the normal variate.
	 * @param mu The mean.
	 * @param sigma The standard deviation.
	 * @return The value of the probability density function at x.
	 */
	double dnorm(double x, double mu = 0, double sigma = 1);
	
	/**
	 * Cumulative probability (CDF) of normal distribution.
	 * @param x A specific value of the normal variate.
	 * @param mu The mean.
	 * @param sigma The standard deviation.
	 * @param eps An Epsilon functor to control accuracy.
	 * @return P(X <= x).
	 */
	double pnorm(double x, double mu = 0, double sigma = 1, const Epsilon & eps = Epsilon());
	
	/**
	 * Quantile of normal distribution.
	 * @param p The cumulative probability.
	 * @param mu The mean.
	 * @param sigma The standard deviation.
	 * @param eps An Epsilon functor to control accuracy.
	 * @return The quantile x makes P(X <= x) == p.
	 */
	double qnorm(double p, double mu = 0, double sigma = 1, const Epsilon & eps = Epsilon());
	
	/**
	 * Probability density of t distribution.
	 * @param x A specific value of the variate.
	 * @param n The degrees of freedom.
	 * @return The value of the probability density function at x.
	 */
	double dt(double x, int n);
	
	/**
	 * Cumulative probability (CDF) of t distribution.
	 * @param x A specific value of the variate.
	 * @param n The degrees of freedom.
	 * @param eps An Epsilon functor to control accuracy.
	 * @return P(X <= x).
	 */
	double pt(double x, int n, const Epsilon & eps = Epsilon());
	
	/**
	 * Quantile of t distribution.
	 * @param p The cumulative probability.
	 * @param n The degrees of freedom.
	 * @param eps An Epsilon functor to control accuracy.
	 * @return The quantile x makes P(X <= x) == p.
	 */
	double qt(double p, int n, const Epsilon & eps = Epsilon());
	
	/**
	 * Probability of poisson distribution.
	 * @param lmd The lambda parameter (> 0).
	 * @param k The occurrence number of specific event (k = 0, 1, 2, ...).
	 * @param cum Calculate P(X <= k) if cum is true, else calculate P(X = k).
	 * @return P(X <= k) if cum is true, else P(X = k).
	 */
	double pois(double lmd, int k, bool cum = false);
	
	/**
	 * Probability of binomial distribution.
	 * @param n The number of independent experiments.
	 * @param p The occurrence probability of specific event in one experiment.
	 * @param k The number of occurrences of the specific event in the n experiments.
	 * @param cum Calculate P(X <= k) if cum is true, else calculate P(X = k).
	 * @return P(X <= k) if cum is true, else P(X = k).
	 */
	double binom(int n, double p, int k, bool cum = false);
	
	/**
	 * Cumulative probability of exponential distribution.
	 * @param theta The parameter of exponential distribution (1/rate).
	 * @param x A specific value of this exponential variate.
	 * @return P(X <= x).
	 */
	double expo(double theta, double x);
	
	/**
	 * Gamma function.
	 * @param x The parameter of gamma function.
	 * @return The value of gamma function.
	 */
	double gamma(double x);
	
	/**
	 * Linear regression.
	 * @param xdata The data of independent variable.
	 * @param ydata The data of dependent variable.
	 * @return A LineParam structure encapsulating the slope and intercept of the regression line.
	 * @see LineParam
	 */
	LineParam linregress(const Vector & xdata, const Vector & ydata);
	
	/**
	 * Generate a vector from a sequence of numbers.
	 * @param from The start point of the sequence.
	 * @param to The end point (may exclude) of the sequence.
	 * @param step The step length of every progress.
	 * @return The vector contains the number sequence.
	 */
	Vector seq(double from, double to, double step = 1);
	
	/**
	 * Generate a vector from a sequence of numbers uniformly spread a linear range.
	 * @param start The start point of the range.
	 * @param end The end point of the range.
	 * @param count The count of numbers in the sequence (including the start and end point).
	 * @return The vector contains the number sequence.
	 */
	Vector linspace(double start, double end, int count = 100);
	
	/**
	 * Find the maximum value of the data.
	 * @param data A vector contains the data.
	 * @return The maximum value.
	 */
	double max(const Vector & data);
	
	/**
	 * Find the minimum value of the data.
	 * @param data A vector contains the data.
	 * @return The minimum value.
	 */
	double min(const Vector & data);
	
	/**
	 * Calculate the sum of a number sequence.
	 * @param data The sequence of numbers.
	 * @return The sum of every number in the sequence.
	 */
	double sum(const Vector & data);
	
	/**
	 * Calculate the product of a number sequence.
	 * @param data The sequence of numbers.
	 * @return The product of every number in the sequence.
	 */
	double prod(const Vector & data);
	
	/**
	 * Add two vectors of same dimension.
	 * @param vec1 The first vector.
	 * @param vec2 The second vector.
	 * @return A vector contains the result.
	 */
	Vector add(const Vector & vec1, const Vector & vec2);
	
	/**
	 * Add a vector and a scalar elementwise.
	 * This function add the scalar to every component of the vector.
	 * @param vec The vector.
	 * @param scalar The scalar value.
	 * @return A vector contains the result.
	 */
	Vector add(const Vector & vec, double scalar);
	
	/**
	 * Add two vectors of same dimension.
	 * @param vec1 The first vector.
	 * @param vec2 The second vector.
	 * @return A vector contains the result.
	 */
	Vector operator+(const Vector & vec1, const Vector & vec2);
	
	/**
	 * Add a vector and a scalar elementwise.
	 * This function add the scalar to every component of the vector.
	 * @param vec The vector.
	 * @param scalar The scalar value.
	 * @return A vector contains the result.
	 */
	Vector operator+(const Vector & vec, double scalar);
	
	/**
	 * Add a vector and a scalar elementwise.
	 * This function add the scalar to every component of the vector.
	 * @param scalar The scalar value.
	 * @param vec The vector.
	 * @return A vector contains the result.
	 */
	Vector operator+(double scalar, const Vector & vec);
	
	/**
	 * Subtract two vectors of same dimension.
	 * @param vec1 The first vector.
	 * @param vec2 The second vector.
	 * @return A vector contains the result.
	 */
	Vector sub(const Vector & vec1, const Vector & vec2);
	
	/**
	 * Subtract a vector and a scalar elementwise.
	 * This function subtract the scalar to every component of the vector.
	 * @param vec The vector.
	 * @param scalar The scalar value.
	 * @param dir If dir is true compute vec - scalar else compute scalar - vec.
	 * @return A vector contains the result.
	 */
	Vector sub(const Vector & vec, double scalar, bool dir = true);
	
	/**
	 * Subtract two vectors of same dimension.
	 * @param vec1 The first vector.
	 * @param vec2 The second vector.
	 * @return A vector contains the result.
	 */
	Vector operator-(const Vector & vec1, const Vector & vec2);
	
	/**
	 * Subtract a vector and a scalar elementwise.
	 * This function subtract the scalar to every component of the vector.
	 * @param vec The vector.
	 * @param scalar The scalar value.
	 * @return A vector contains the result.
	 */
	Vector operator-(const Vector & vec, double scalar);
	
	/**
	 * Subtract a vector and a scalar elementwise.
	 * This function subtract the scalar to every component of the vector.
	 * @param scalar The scalar value.
	 * @param vec The vector.
	 * @return A vector contains the result.
	 */
	Vector operator-(double scalar, const Vector & vec);
	
	/**
	 * Multiply two vectors of same dimension elementwise.
	 * @param vec1 The first vector.
	 * @param vec2 The second vector.
	 * @return A vector contains the result.
	 */
	Vector mul(const Vector & vec1, const Vector & vec2);
	
	/**
	 * Multiply a vector and a scalar elementwise.
	 * @param vec The vector.
	 * @param scalar The scalar value.
	 * @return A vector contains the result.
	 */
	Vector mul(const Vector & vec, double scalar);
	
	/**
	 * Multiply two vectors of same dimension elementwise.
	 * @param vec1 The first vector.
	 * @param vec2 The second vector.
	 * @return A vector contains the result.
	 */
	Vector operator*(const Vector & vec1, const Vector & vec2);
	
	/**
	 * Multiply a vector and a scalar elementwise.
	 * @param vec The vector.
	 * @param scalar The scalar value.
	 * @return A vector contains the result.
	 */
	Vector operator*(const Vector & vec, double scalar);
	
	/**
	 * Multiply a vector and a scalar elementwise.
	 * @param scalar The scalar value.
	 * @param vec The vector.
	 * @return A vector contains the result.
	 */
	Vector operator*(double scalar, const Vector & vec);
	
	/**
	 * Divide two vectors of same dimension elementwise.
	 * @param vec1 The first vector.
	 * @param vec2 The second vector.
	 * @return A vector contains the result.
	 */
	Vector div(const Vector & vec1, const Vector & vec2);
	
	/**
	 * Divide a vector and a scalar elementwise.
	 * @param vec The vector.
	 * @param scalar The scalar value.
	 * @param dir If dir is true compute vec / scalar else compute scalar / vec.
	 * @return A vector contains the result.
	 */
	Vector div(const Vector & vec, double scalar, bool dir = true);
	
	/**
	 * Divide two vectors of same dimension elementwise.
	 * @param vec1 The first vector.
	 * @param vec2 The second vector.
	 * @return A vector contains the result.
	 */
	Vector operator/(const Vector & vec1, const Vector & vec2);
	
	/**
	 * Divide a vector and a scalar elementwise.
	 * @param vec The vector.
	 * @param scalar The scalar value.
	 * @return A vector contains the result.
	 */
	Vector operator/(const Vector & vec, double scalar);
	
	/**
	 * Divide a vector and a scalar elementwise.
	 * @param scalar The scalar value.
	 * @param vec The vector.
	 * @return A vector contains the result.
	 */
	Vector operator/(double scalar, const Vector & vec);
	
	/**
	 * Dot product of two vectors.
	 * The dot product of (a1, a2, a3) and (b1, b2, b3) is a1*b1 + a2*b2 + a3*b3.
	 * @param vec1 The first vector.
	 * @param vec2 The second vector.
	 * @return The dot product of these two vectors.
	 */
	double dot_prod(const Vector & vec1, const Vector & vec2);
	
	/**
	 * Cross product of two 3-dimensional vectors.
	 * @param vec1 The first 3-dimensional vector.
	 * @param vec2 The second 3-dimensional vector.
	 * @return A 3-dimensional vector represents the cross product of these two vectors.
	 */
	Vector cross_prod(const Vector & vec1, const Vector & vec2);
	
	/**
	 * The Euclidean norm (length) of a vector.
	 * @param vec The vector.
	 * @return The Euclidean norm.
	 */
	double norm(const Vector & vec);
	
	/**
	 * Generate the string representation of a vector.
	 * @param vec The vector.
	 * @param sep The separator string.
	 * @param delim The delimiter string.
	 * @return The string representation of the vector.
	 */
	string to_str(const Vector & vec, string sep = " ", string delim = "()");
	
	/**
	 * Make a vector.
	 * The components must be double type, so integers should add a trailing dot.@n
	 * To make a vector (1 2 9): <code>make_vec(3, 1., 2., 9.)</code>
	 * @param n The number of components.
	 * @param ... The components (double type) of the vector.
	 * @return A vector contains the specific components.
	 */
	Vector make_vec(int n, ...);
	
	/**
	 * Load data from an input stream.
	 * @param ins The input stream.
	 * @param delim The delimiter string.
	 * @return A vector contains the data from the input stream.
	 */
	Vector load(istream & ins, string delim = " ");
	
	/**
	 * Load table/matrix data from an input stream.
	 * @param ins The input stream.
	 * @param nrow The number of rows.
	 * @param ncol The number of columns.
	 * @param delim The delimiter string.
	 * @return A vector of vector contains the data from the input stream.
	 */
	Table load(istream & ins, int nrow, int ncol, string delim = " ");
	
	/**
	 * Save data to an output stream.
	 * @param outs The output stream.
	 * @param data The data to be saved.
	 * @param delim The delimiter string.
	 */
	void save(ostream & outs, const Vector & data, string delim = " ");
	
	/**
	 * Save table/matrix data to an output stream.
	 * @param outs The output stream.
	 * @param data The data to be saved.
	 * @param delim The delimiter string.
	 */
	void save(ostream & outs, const Table & data, string delim = " ");
	
	/**
	 * Stream insertion operator for vector.
	 * @param outs The output stream.
	 * @param vec The vector.
	 * @return The reference of the output stream.
	 */
	ostream & operator<<(ostream & outs, const Vector & vec);
	
	/**
	 * Stream extraction operator for vector.
	 * @param ins The input stream.
	 * @param vec The vector.
	 * @return The reference of the input stream.
	 */
	istream & operator>>(istream & ins, Vector & vec);
	
	/**
	 * Floating point random number in rang [low, high] inclusive.
	 * @param low The low boundary of the range.
	 * @param high The high boundary of the range.
	 * @return A floating point random number in range [low, high] inclusive.
	 */
	double randf(double low, double high);
	
	/**
	 * Floating point random numbers in rang [low, high] inclusive.
	 * @param low The low boundary of the range.
	 * @param high The high boundary of the range.
	 * @param n The count of random numbers to generate.
	 * @return A vector contains random numbers in range [low, high] inclusive.
	 */
	Vector randf(double low, double high, int n);
	
	/**
	 * Integer random number in range [low, high] inclusive.
	 * @param low The low boundary of the range.
	 * @param high The high boundary of the range.
	 * @return An integer random number in range [low, high] inclusive.
	 */
	int randi(int low, int high);
	
	/**
	 * Integer random numbers in range [low, high] inclusive.
	 * @param low The low boundary of the range.
	 * @param high The high boundary of the range.
	 * @param n The count of random numbers to generate.
	 * @return A vector contains integer random numbers in range [low, high] inclusive.
	 */
	Vector randi(int low, int high, int n);
	
	/**
	 * 0~1 distribution.
	 * This function simulate a boolean random variable whose probability to be true is p.
	 * So if you set p to 0.5 and run this function 100 times, the times it returns true is approximately 50.
	 * @param p The probability between 0 and 1.
	 * @return A boolean random value. It's probability to be true is p.
	 */
	bool randp(double p);
	
	/**
	 * Set seed for pseudo-random number generator.
	 * This function set the seed according to the current system date-time.
	 */
	void randseed();
	
	/**
	 * Set seed for pseudo-random number generator.
	 * @param s The seed.
	 */
	void randseed(unsigned int s);
	
	/**
	 * Random number of normal distribution.
	 * @param mu The mean.
	 * @param sigma The standard deviation.
	 * @return A random number of normal distribution.
	 */
	double rnorm(double mu = 0, double sigma = 1);
	
	/**
	 * Random numbers of normal distribution.
	 * @param n The count of random numbers.
	 * @param mu The mean.
	 * @param sigma The standard deviation.
	 * @return A vector contains random numbers of normal distribution.
	 */
	Vector rnorm(int n, double mu = 0, double sigma = 1);
	
	/**
	 * Random number of poisson distribution.
	 * @param lmd The lambda parameter (> 0).
	 * @return An integer random number of poisson distribution.
	 */
	int rpois(double lmd);
	
	/**
	 * Random numbers of poisson distribution.
	 * @param lmd The lambda parameter (> 0).
	 * @param n The count of random numbers.
	 * @return A vector contains random numbers of poisson distribution.
	 */
	Vector rpois(double lmd, int n);
	
	/**
	 * Random number of binomial distribution.
	 * @param n The number of independent experiments.
	 * @param p The occurrence probability of specific event in one experiment.
	 * @return An integer random number of binomial distribution.
	 */
	int rbinom(int n, double p);
	
	/**
	 * Random numbers of binomial distribution.
	 * @param n The number of independent experiments.
	 * @param p The occurrence probability of specific event in one experiment.
	 * @param count The count of random numbers.
	 * @return A vector contains random numbers of binomial distribution.
	 */
	Vector rbinom(int n, double p, int count);
	
	/**
	 * Random number of exponential distribution.
	 * @param theta The parameter of exponential distribution (1/rate).
	 * @return A random number of exponential distribution.
	 */
	double rexp(double theta);
	
	/**
	 * Random numbers of exponential distribution.
	 * @param theta The parameter of exponential distribution (1/rate).
	 * @param n The count of random numbers.
	 * @return A vector contains random numbers of exponential distribution.
	 */
	Vector rexp(double theta, int n);
	
	/**
	 * Round a number to specific decimal digits.
	 * @param num The number to round.
	 * @param ndec The number of decimal digits.
	 * @return A number with specific decimal digitals.
	 */
	double prec(double num, int ndec = 2);
	
	/**
	 * Round every component of a vector to specific decimal digits.
	 * @param vec The vector.
	 * @param ndec The number of decimal digits.
	 * @return A vector contains the rounded data.
	 */
	Vector prec(const Vector & vec, int ndec = 4);
	
	/**
	 * Factorial of integer n (n!).
	 * @param n The integer number.
	 * @return The factorial (n!).
	 */
	double fac(unsigned int n);
	
	/**
	 * Permutation number (mPn).
	 * mPn = m! / (m - n)!
	 * @param m The total number of all probable events.
	 * @param n The selected number of events.
	 * @return The permutation number.
	 */
	double perm(unsigned int m, unsigned int n);
	
	/**
	 * Combination number (mCn).
	 * mCn = mPn / n!
	 * @param m The total number of all probable events.
	 * @param n The selected number of events.
	 * @return The combination number.
	 */
	double comb(unsigned int m, unsigned int n);
	
	/**
	 * Greatest common divisor.
	 * @param a An integer.
	 * @param b Another integer.
	 * @return The greatest common divisor of these two integers.
	 */
	unsigned int gcd(unsigned int a, unsigned int b);
	
	/**
	 * Least common multiple.
	 * @param a An integer.
	 * @param b Another integer.
	 * @return The least common multiple of these two integers.
	 */
	unsigned int lcm(unsigned int a, unsigned int b);
	
	/**
	 * Apply a function to each component of a vector.
	 * @param func The function to apply (double func(double)).
	 * @param data The vector.
	 * @return A vector contains the results.
	 */
	template<typename Func>
	Vector each(Func func, const Vector & data);
	
	/**
	 * Filter a vector of data.
	 * @param func The filter function (bool func(double)).
	 * @param data The vector of data to filter.
	 * @return A vector contains the data that make the filter function returns true.
	 */
	template<typename Func>
	Vector filter(Func func, const Vector & data);
	
	/**
	 * Find a region contains at least one root.
	 * @param func A function represents the equation func(x) == 0.
	 * @param guess A guessed region as input, the real region as output if the probe succeeds.
	 * @param factor The scale factor used to adjust the region in every iteration.
	 * @param ntry The maximum times to be tried.
	 * @return A boolean value indicates whether the probe succeeds.
	 */
	template<typename Func>
	bool probe(Func func, Pair & guess, double factor = 1.6, int ntry = 50);
	
	/**
	 * Scan a specific scope to find sub-regions contain root.
	 * @param func A function represents the equation func(x) == 0.
	 * @param scope The scope to scan.
	 * @param nr The number of sub-regions by which the scope will be sliced.
	 * @return A vector holds the sub-regions contain root.
	 */
	template<typename Func>
	Pairs scan(Func func, Pair scope, int nr);
	
	/**
	 * Find a root in a specific region using binary search algorithm.
	 * @param func A function represents the equation func(x) == 0.
	 * @param region The region to search.
	 * @param eps An Epsilon functor to control accuracy.
	 * @return The root found.
	 */
	template<typename Func>
	double solve(Func func, Pair region, const Epsilon & eps = Epsilon());
	
	/**
	 * Numerical integration.
	 * @param func The integrand function (double func(double)).
	 * @param region The definite integral region.
	 * @param eps An Epsilon functor to control accurary.
	 * @return The definite integration.
	 */
	template<typename Func>
	double integrate(Func func, Pair region, const Epsilon & eps = Epsilon());
	
	/**
	 * Convert a C array to a C++ vector.
	 * @param data The array contains the data.
	 * @param len The length/size (number of elements) of the array.
	 * @return A vector contains the same data of the array.
	 */
	template<typename T>
	Vector a2vec(T data[], int len);
	
	#include "template_impl.cpp"
}

#endif
