template<typename Func>
Vector each(Func func, const Vector & data) {
	Vector vec;
	Vector::const_iterator citer;
	for (citer = data.begin(); citer != data.end(); ++citer) vec.push_back(func(*citer));
	return vec;
}

template<typename Func>
Vector filter(Func func, const Vector & data) {
	Vector result;
	Vector::const_iterator citer;
	for (citer = data.begin(); citer != data.end(); ++citer)
		if (func(*citer)) result.push_back(*citer);
	return result;
}

template<typename Func>
bool probe(Func func, Pair & guess, double factor, int ntry) {
	double lv = func(guess.first);
	double rv = func(guess.second);
	while (ntry > 0) {
		if (lv * rv <= 0) return true;
		if (abs(lv) < abs(rv)) {
			guess.first -= factor * (guess.second - guess.first);
			lv = func(guess.first);
		}
		else {
			guess.second += factor * (guess.second - guess.first);
			rv = func(guess.second);
		}
		--ntry;
	}
	return false;
}

template<typename Func>
Pairs scan(Func func, Pair scope, int nr) {
	Pairs vec;
	Vector marks = linspace(scope.first, scope.second, nr + 1);
	for (int i = 0; i < nr; ++i) {
		if (func(marks[i]) * func(marks[i+1]) <= 0) vec.push_back(make_pair(marks[i], marks[i+1]));
	}
	return vec;
}

template<typename Func>
double solve(Func func, Pair region, const Epsilon & eps) {
	double lv = region.first;
	double rv = region.second;
	if (func(lv) == 0) return lv;
	if (func(rv) == 0) return rv;
	while (true) {
		double mid = (lv + rv) / 2;
		if (eps(rv - lv) || func(mid) == 0) return mid;
		if (func(lv) * func(mid) < 0) rv = mid;
		else lv = mid;
	}
}

template<typename Func>
double integrate(Func func, Pair region, const Epsilon & eps) {
	int n = static_cast<int>((region.second - region.first) * 2);
	if (n < 2) n = 2;
	Vector marks = linspace(region.first, region.second, n + 1);
	double s1 = func(marks[0]) + func(marks[n]);
	double s2 = 0;
	for (int i = 1; i < n; ++i) s2 += func(marks[i]);
	s2 *= 2;
	double s4 = 0;
	for (int i = 0; i < n; ++i) s4 += func((marks[i] + marks[i+1]) / 2);
	s4 *= 4;
	double s = (region.second - region.first) / n / 6 * (s1 + s2 + s4);
	int k = 2;
	while (true) {
		n *= k;
		marks = linspace(region.first, region.second, n + 1);
		s1 = func(marks[0]) + func(marks[n]);
		s2 = 0;
		for (int i = 1; i < n; ++i) s2 += func(marks[i]);
		s2 *= 2;
		s4 = 0;
		for (int i = 0; i < n; ++i) s4 += func((marks[i] + marks[i+1]) / 2);
		s4 *= 4;
		double sn = (region.second - region.first) / n / 6 * (s1 + s2 + s4);
		if (eps(sn - s)) {
			s = sn;
			break;
		}
		s = sn;
		k += 1;
	}
	return s;
}

template<typename T>
Vector a2vec(T data[], int len) {
	Vector vec;
	for (int i = 0; i < len; ++i) vec.push_back(data[i]);
	return vec;
}
