#include <ctime>
#include <cstdlib>
#include <cstdarg>
#include <climits>
#include <sstream>
#include <numeric>
#include <stdexcept>
#include <algorithm>
#include <functional>

#include "mathkit.hpp"

using namespace std;

namespace mathkit {
	bool Epsilon::operator()(double val) const {
		if (_eps == 0) return val == 0;
		else return abs(val) < _eps;
	}
	
	double pv(double amount, double rate, double period) {
		return amount / pow(1 + rate, period);
	}
	
	double fv(double amount, double rate, double period) {
		return amount * pow(1 + rate, period);
	}
	
	double pv_coef(double rate, double period) {
		return pow(1 + rate, -period);
	}
	
	double fv_coef(double rate, double period) {
		return pow(1 + rate, period);
	}
	
	double apv(double annuity, double rate, int period, bool prepaid) {
		return annuity * apv_coef(rate, period, prepaid);
	}
	
	double afv(double annuity, double rate, int period, bool prepaid) {
		return annuity * afv_coef(rate, period, prepaid);
	}
	
	double apv_coef(double rate, int period, bool prepaid) {
		if (rate == 0) return period;
		double coef = (1 - pow(1 + rate, -period)) / rate;
		if (prepaid) coef *= (1 + rate);
		return coef;
	}
	
	double afv_coef(double rate, int period, bool prepaid) {
		if (rate == 0) return period;
		double coef = (pow(1 + rate, period) - 1) / rate;
		if (prepaid) coef *= (1 + rate);
		return coef;
	}
	
	double spv(const Vector & amount, double rate, bool prepaid) {
		double pvalue = 0;
		int period = 1;
		Vector::const_iterator citer = amount.begin();
		if (prepaid) {
			pvalue += *citer;
			++citer;
		}
		while (citer != amount.end()) {
			pvalue += pv(*citer, rate, period);
			++period;
			++citer;
		}
		return pvalue;
	}
	
	double sfv(const Vector & amount, double rate, bool prepaid) {
		double fvalue = 0;
		int period = 1;
		Vector::const_reverse_iterator criter = amount.rbegin();
		if (!prepaid) {
			fvalue += *criter;
			++criter;
		}
		while (criter != amount.rend()) {
			fvalue += fv(*criter, rate, period);
			++period;
			++criter;
		}
		return fvalue;
	}
	
	double comp_rate(double pval, double fval, double period) {
		return pow(fval / pval, 1 / period) - 1;
	}
	
	double mean(const Vector & data) {
		double sum = 0;
		Vector::const_iterator citer;
		for (citer = data.begin(); citer != data.end(); ++citer) sum += *citer;
		return sum / data.size();
	}
	
	double median(const Vector & data, bool sorted) {
		if (!sorted) {
			Vector temp = data;
			sort(temp.begin(), temp.end());
			return median(temp, true);
		}
		if (data.size() % 2 == 0) return (data[data.size() / 2] + data[data.size() / 2 - 1]) / 2;
		else return data[data.size() / 2];
	}
	
	double var(const Vector & data, bool sample) {
		double avg = mean(data);
		double dev = 0;
		Vector::const_iterator citer;
		for (citer = data.begin(); citer != data.end(); ++citer) dev += (*citer - avg) * (*citer - avg);
		if (sample) return dev / (data.size() - 1);
		else return dev / data.size();
	}
	
	double sd(const Vector & data, bool sample) {
		return sqrt(var(data, sample));
	}
	
	double cov(const Vector & data1, const Vector & data2, bool sample) {
		double avg1 = mean(data1);
		double avg2 = mean(data2);
		double dev = 0;
		Vector::const_iterator citer1, citer2;
		citer1 = data1.begin();
		citer2 = data2.begin();
		while (citer1 != data1.end() && citer2 != data2.end()) {
			dev += (*citer1 - avg1) * (*citer2 - avg2);
			++citer1;
			++citer2;
		}
		if (sample) return dev / (data1.size() - 1);
		else return dev / data1.size();
	}
	
	double cor(const Vector & data1, const Vector & data2) {
		return cov(data1, data2) / (sd(data1) * sd(data2));
	}
	
	double moment(const Vector & data, int k, bool central) {
		double result = 0;
		Vector::const_iterator citer;
		if (central) {
			double mu = mean(data);
			for (citer = data.begin(); citer != data.end(); ++citer)
				result += pow(*citer - mu, k);
		}
		else {
			for (citer = data.begin(); citer != data.end(); ++citer)
				result += pow(*citer, k);
		}
		return result / data.size();
	}
	
	double skew(const Vector & data) {
		return moment(data, 3) / pow(moment(data, 2), 1.5);
	}
	
	double kurt(const Vector & data, bool excess) {
		double result = moment(data, 4) / pow(moment(data, 2), 2);
		if (excess) result -= 3;
		return result;
	}
	
	double dnorm(double x, double mu, double sigma) {
		double z = (x - mu) / sigma;
		return 1 / (sqrt(2 * pi) * sigma) * exp(-0.5 * z * z);
	}
	
	double pnorm(double x, double mu, double sigma, const Epsilon & eps) {
		double z = (x - mu) / sigma;
		double d = dnorm(z);
		double s = z, sn = z;
		double p = 0.5 + d * s;
		int i = 3;
		while (true) {
			sn *= z * z / i;
			s += sn;
			double pn = 0.5 + d * s;
			if (eps(pn - p)) {
				p = pn;
				break;
			}
			p = pn;
			i += 2;
		}
		return p;
	}
	
	double qnorm(double p, double mu, double sigma, const Epsilon & eps) {
		Epsilon peps(1e-15);
		if (peps(p - 0.5)) return mu;
		double lx, rx;
		if (p > 0.5) {
			rx = sigma;
			while (pnorm(rx, mu, sigma, peps) < p) rx += sigma;
			lx = rx - sigma;
		}
		else {
			lx = -sigma;
			while (pnorm(lx, mu, sigma, peps) > p) lx -= sigma;
			rx = lx + sigma;
		}
		if (peps(p - pnorm(lx, mu, sigma, peps))) return lx;
		if (peps(p - pnorm(rx, mu, sigma, peps))) return rx;
		while (true) {
			double mid = (lx + rx) / 2;
			double pmid = pnorm(mid, mu, sigma, peps);
			if (eps(rx - lx) || peps(p - pmid)) return mid;
			if (pmid > p) rx = mid;
			else lx = mid;
		}
	}
	
	double dt(double x, int n) {
		return gamma((n + 1) / 2.0) / (sqrt(n * pi) * gamma(n / 2.0)) * pow(1 + x * x / n, (n + 1) / -2.0);
	}
	
	double pt(double x, int n, const Epsilon & eps) {
		if (x == 0) return 0.5;
		if (x > 0) {
			Pair region(-x, x);
			double p = integrate(bind2nd(ptr_fun<double, int, double>(dt), n), region, eps);
			return p + (1 - p) / 2;
		}
		else {
			Pair region(x, -x);
			double p = integrate(bind2nd(ptr_fun<double, int, double>(dt), n), region, eps);
			return (1 - p) / 2;
		}
	}
	
	double qt(double p, int n, const Epsilon & eps) {
		Epsilon peps(1e-10);
		if (peps(p - 0.5)) return 0;
		double lx, rx;
		if (p > 0.5) {
			rx = 1;
			while (pt(rx, n, peps) < p) rx += 1;
			lx = rx - 1;
		}
		else {
			lx = -1;
			while (pt(lx, n, peps) > p) lx -= 1;
			rx = lx + 1;
		}
		if (peps(p - pt(lx, n, peps))) return lx;
		if (peps(p - pt(rx, n, peps))) return rx;
		while (true) {
			double mid = (lx + rx) / 2;
			double pmid = pt(mid, n, peps);
			if (eps(rx - lx) || peps(p - pmid)) return mid;
			if (pmid > p) rx = mid;
			else lx = mid;
		}
	}
	
	double pois(double lmd, int k, bool cum) {
		if (!cum) return pow(lmd, k) * exp(-lmd) / fac(k);
		double result = 0;
		for (int i = 0; i <= k; ++i) result += pois(lmd, i, false);
		return result;
	}
	
	double binom(int n, double p, int k, bool cum) {
		if (!cum) return comb(n, k) * pow(p, k) * pow(1 - p, n - k);
		double result = 0;
		for (int i = 0; i <= k; ++i) result += binom(n, p, i, false);
		return result;
	}
	
	double expo(double theta, double x) {
		if (x > 0) return 1 - exp(-1 / theta * x);
		else return 0;
	}
	
	double gamma(double x) {
		if (!(x > 0)) throw invalid_argument("Invalid argument.");
		double n = x - 1;
		double ipart;
		if (modf(n, &ipart) == 0 && n <= UINT_MAX) return fac((unsigned int) n);
		double p[] = {1.000000000190015, 76.18009172947146, -86.50532032941677, 24.01409824083091, -1.231739572450155, 1.208650973866179e-3, -5.395239384953e-6};
		double s = p[0];
		for (int i = 1; i <= 6; ++i) s += p[i] / (x + i);
		return sqrt(2 * pi) / x * s * pow(x + 5.5, x + 0.5) * exp(-x - 5.5);
	}
	
	LineParam linregress(const Vector & xdata, const Vector & ydata) {
		LineParam regline;
		regline.slope = cov(xdata, ydata) / var(xdata);
		regline.intercept = mean(ydata) - regline.slope * mean(xdata);
		return regline;
	}
	
	Vector seq(double from, double to, double step) {
		Vector vec;
		if (step > 0) {
			if (from > to) return vec;
			do {
				vec.push_back(from);
				from += step;
			} while (!(from > to));
		}
		else if (step < 0) {
			if (from < to) return vec;
			do {
				vec.push_back(from);
				from += step;
			} while (!(from < to));
		}
		return vec;
	}
	
	Vector linspace(double start, double end, int count) {
		Vector vec;
		if (count < 2) return vec;
		if (count == 2) {
			vec.push_back(start);
			vec.push_back(end);
		}
		else {
			double delta = (end - start) / (count - 1);
			vec.push_back(start);
			for (int i = 0; i < count - 2; ++i) {
				start += delta;
				vec.push_back(start);
			}
			vec.push_back(end);
		}
		return vec;
	}
		
	double max(const Vector & data) {
		return *(max_element(data.begin(), data.end()));
	}
	
	double min(const Vector & data) {
		return *(min_element(data.begin(), data.end()));
	}
	
	double sum(const Vector & data) {
		return accumulate(data.begin(), data.end(), 0.0);
	}
	
	double prod(const Vector & data) {
		return accumulate(data.begin(), data.end(), 1.0, multiplies<double>());
	}
	
	Vector add(const Vector & vec1, const Vector & vec2) {
		Vector result;
		Vector::const_iterator citer1, citer2;
		citer1 = vec1.begin();
		citer2 = vec2.begin();
		while (citer1 != vec1.end() && citer2 != vec2.end()) {
			result.push_back(*citer1 + *citer2);
			++citer1;
			++citer2;
		}
		return result;
	}
	
	Vector add(const Vector & vec, double scalar) {
		Vector result;
		Vector::const_iterator citer;
		for (citer = vec.begin(); citer != vec.end(); ++citer) result.push_back(*citer + scalar);
		return result;
	}
	
	Vector operator+(const Vector & vec1, const Vector & vec2) {
		return add(vec1, vec2);
	}
	
	Vector operator+(const Vector & vec, double scalar) {
		return add(vec, scalar);
	}
	
	Vector operator+(double scalar, const Vector & vec) {
		return add(vec, scalar);
	}
	
	Vector sub(const Vector & vec1, const Vector & vec2) {
		Vector result;
		Vector::const_iterator citer1, citer2;
		citer1 = vec1.begin();
		citer2 = vec2.begin();
		while (citer1 != vec1.end() && citer2 != vec2.end()) {
			result.push_back(*citer1 - *citer2);
			++citer1;
			++citer2;
		}
		return result;
	}
	
	Vector sub(const Vector & vec, double scalar, bool dir) {
		Vector result;
		Vector::const_iterator citer;
		for (citer = vec.begin(); citer != vec.end(); ++citer) {
			double element = *citer - scalar;
			if (!dir) element = -element;
			result.push_back(element);
		}
		return result;
	}
	
	Vector operator-(const Vector & vec1, const Vector & vec2) {
		return sub(vec1, vec2);
	}
	
	Vector operator-(const Vector & vec, double scalar) {
		return sub(vec, scalar);
	}
	
	Vector operator-(double scalar, const Vector & vec) {
		return sub(vec, scalar, false);
	}
	
	Vector mul(const Vector & vec1, const Vector & vec2) {
		Vector result;
		Vector::const_iterator citer1, citer2;
		citer1 = vec1.begin();
		citer2 = vec2.begin();
		while (citer1 != vec1.end() && citer2 != vec2.end()) {
			result.push_back(*citer1 * *citer2);
			++citer1;
			++citer2;
		}
		return result;
	}
	
	Vector mul(const Vector & vec, double scalar) {
		Vector result;
		Vector::const_iterator citer;
		for (citer = vec.begin(); citer != vec.end(); ++citer) result.push_back(*citer * scalar);
		return result;
	}
	
	Vector operator*(const Vector & vec1, const Vector & vec2) {
		return mul(vec1, vec2);
	}
	
	Vector operator*(const Vector & vec, double scalar) {
		return mul(vec, scalar);
	}
	
	Vector operator*(double scalar, const Vector & vec) {
		return mul(vec, scalar);
	}
	
	Vector div(const Vector & vec1, const Vector & vec2) {
		Vector result;
		Vector::const_iterator citer1, citer2;
		citer1 = vec1.begin();
		citer2 = vec2.begin();
		while (citer1 != vec1.end() && citer2 != vec2.end()) {
			result.push_back(*citer1 / *citer2);
			++citer1;
			++citer2;
		}
		return result;
	}
	
	Vector div(const Vector & vec, double scalar, bool dir) {
		Vector result;
		Vector::const_iterator citer;
		for (citer = vec.begin(); citer != vec.end(); ++citer) {
			if (dir) result.push_back(*citer / scalar);
			else result.push_back(scalar / *citer);
		}
		return result;
	}
	
	Vector operator/(const Vector & vec1, const Vector & vec2) {
		return div(vec1, vec2);
	}
	
	Vector operator/(const Vector & vec, double scalar) {
		return div(vec, scalar);
	}
	
	Vector operator/(double scalar, const Vector & vec) {
		return div(vec, scalar, false);
	}
	
	double dot_prod(const Vector & vec1, const Vector & vec2) {
		double result = 0;
		Vector::const_iterator citer1, citer2;
		citer1 = vec1.begin();
		citer2 = vec2.begin();
		while (citer1 != vec1.end() && citer2 != vec2.end()) {
			result += *citer1 * *citer2;
			++citer1;
			++citer2;
		}
		return result;
	}
	
	Vector cross_prod(const Vector & vec1, const Vector & vec2) {
		Vector vec;
		vec.push_back(vec1[1] * vec2[2] - vec1[2] * vec2[1]);
		vec.push_back(vec1[2] * vec2[0] - vec1[0] * vec2[2]);
		vec.push_back(vec1[0] * vec2[1] - vec1[1] * vec2[0]);
		return vec;
	}
	
	double norm(const Vector & vec) {
		return sqrt(dot_prod(vec, vec));
	}
	
	string to_str(const Vector & vec, string sep, string delim) {
		ostringstream oss;
		Vector::size_type n = vec.size();
		Vector::size_type last = n - 1;
		for (Vector::size_type i = 0; i < n; ++i) {
			oss << vec[i];
			if (i != last) oss << sep;
		}
		string result = oss.str();
		if (delim.size() == 2) result = delim[0] + result + delim[1];
		return result;
	}
	
	Vector make_vec(int n, ...) {
		Vector vec;
		va_list argptr;
		va_start(argptr, n);
		for (int i = 0; i < n; ++i) vec.push_back(va_arg(argptr, double));
		va_end(argptr);
		return vec;
	}
	
	Vector load(istream & ins, string delim) {
		Vector data;
		string line;
		if (getline(ins, line)) {
			if (delim != " ") {
				string::size_type i;
				while ((i = line.find(delim)) != string::npos) line.replace(i, delim.size(), " ");
			}
			istringstream iss(line);
			while (!iss.eof()) {
				double item;
				iss >> item;
				data.push_back(item);
			}
		}
		return data;
	}
	
	Table load(istream & ins, int nrow, int ncol, string delim) {
		Table data(ncol, Vector());
		for (int i = 0; i < nrow; ++i) {
			Vector row = load(ins, delim);
			for (int j = 0; j < ncol; ++j) data[j].push_back(row[j]);
		}
		return data;
	}
	
	void save(ostream & outs, const Vector & data, string delim) {
		Vector::const_iterator citer, last;
		last = data.end() - 1;
		int prec = outs.precision();
		outs.precision(15);
		for (citer = data.begin(); citer != data.end(); ++citer) {
			outs << *citer;
			if (citer != last) outs << delim;
		}
		outs << endl;
		outs.precision(prec);
	}
	
	void save(ostream & outs, const Table & data, string delim) {
		int nrow = data[0].size();
		int ncol = data.size();
		for (int i = 0; i < nrow; ++i) {
			Vector row;
			for (int j = 0; j < ncol; ++j) row.push_back(data[j][i]);
			save(outs, row, delim);
		}
	}
	
	ostream & operator<<(ostream & outs, const Vector & vec) {
		save(outs, vec);
		return outs;
	}
	
	istream & operator>>(istream & ins, Vector & vec) {
		vec = load(ins);
		return ins;
	}
	
	double randf(double low, double high) {
		return rand() / (double) RAND_MAX * (high - low) + low;
	}
	
	Vector randf(double low, double high, int n) {
		Vector result;
		for (int i = 0; i < n; ++i) result.push_back(randf(low, high));
		return result;
	}
	
	int randi(int low, int high) {
		return (int) floor(rand() / ((double) RAND_MAX + 1) * ((double) high - low + 1) + low);
	}
	
	Vector randi(int low, int high, int n) {
		Vector result;
		for (int i = 0; i < n; ++i) result.push_back(randi(low, high));
		return result;
	}
	
	bool randp(double p) {
		return !(randf(0, 1) > p);
	}
	
	void randseed() {
		srand((unsigned int) time(NULL));
	}
	
	void randseed(unsigned int s) {
		srand(s);
	}
	
	double rnorm(double mu, double sigma) {
		double u = randf(0, 1);
		double v = randf(0, 1);
		double z = sqrt(-2 * log(u)) * cos(2 * pi * v);
		return mu + sigma * z;
	}
	
	Vector rnorm(int n, double mu, double sigma) {
		Vector result;
		for (int i = 0; i < n; ++i) result.push_back(rnorm(mu, sigma));
		return result;
	}
	
	int rpois(double lmd) {
		double L = exp(-lmd);
		double p = 1;
		int k = 0;
		do {
			k += 1;
			p *= randf(0, 1);
		} while (p > L);
		return k - 1;
	}
	
	Vector rpois(double lmd, int n) {
		Vector result;
		for (int i = 0; i < n; ++i) result.push_back(rpois(lmd));
		return result;
	}
	
	int rbinom(int n, double p) {
		int k = 0;
		for (int i = 0; i < n; ++i)
			if (randp(p)) k += 1;
		return k;
	}
	
	Vector rbinom(int n, double p, int count) {
		Vector result;
		for (int i = 0; i < count; ++i) result.push_back(rbinom(n, p));
		return result;
	}
	
	double rexp(double theta) {
		return -theta * log(randf(0, 1));
	}
	
	Vector rexp(double theta, int n) {
		Vector result;
		for (int i = 0; i < n; ++i) result.push_back(rexp(theta));
		return result;
	}
	
	double prec(double num, int ndec) {
		double intpart;
		double decpart;
		decpart = modf(num, &intpart);
		stringstream ss;
		ss.precision(ndec + 1);
		ss << decpart + 1;
		ss >> decpart;
		return intpart + decpart - 1;
	}
	
	Vector prec(const Vector & vec, int ndec) {
		Vector result;
		Vector::const_iterator citer;
		for (citer = vec.begin(); citer != vec.end(); ++citer)
			result.push_back(prec(*citer, ndec));
		return result;
	}
	
	double fac(unsigned int n) {
		if (n == 0) return 1;
		else {
			double result = 1;
			while (n > 0) {
				result *= n;
				--n;
			}
			return prec(result, 0);
		}
	}
	
	double perm(unsigned int m, unsigned int n) {
		double result = 1;
		while (n > 0) {
			result *= m;
			--m;
			--n;
		}
		return prec(result, 0);
	}
	
	double comb(unsigned int m, unsigned int n) {
		if (m - n < n) return comb(m, m - n);
		return prec(perm(m, n) / fac(n), 0);
	}
	
	unsigned int gcd(unsigned int a, unsigned int b) {
		while (b != 0) {
			int temp = b;
			b = a % b;
			a = temp;
		}
		return a;
	}
	
	unsigned int lcm(unsigned int a, unsigned int b) {
		return a * b / gcd(a, b);
	}
}
