import math, random, numbers, copy

class Epsilon:
    """Epsilon functor (test whether |x| < eps)."""
    def __init__(self, eps = 1e-6):
        """Construct an Epsilon object."""
        self.eps = eps

    def __call__(self, val):
        """Test whether |val| < eps."""
        if self.eps == 0: return val == 0
        else: return abs(val) < self.eps

def pv(amount, rate, period):
    """Present value of compound interest."""
    return amount / (1 + rate) ** period

def fv(amount, rate, period):
    """Future value of compound interest."""
    return amount * (1 + rate) ** period

def pv_coef(rate, period):
    """Present value coefficient of compound interest."""
    return (1 + rate) ** -period

def fv_coef(rate, period):
    """Future value coefficient of compound interest."""
    return (1 + rate) ** period

def apv_coef(rate, period, prepaid = False):
    """Present value coefficient of annuity."""
    if rate == 0: return period
    coef = (1 - (1 + rate) ** -period) / rate
    if prepaid: coef *= (1 + rate)
    return coef

def apv(annuity, rate, period, prepaid = False):
    """Present value of annuity."""
    return annuity * apv_coef(rate, period, prepaid)

def afv_coef(rate, period, prepaid = False):
    """Future value coefficient of annuity."""
    if rate == 0: return period
    coef = ((1 + rate) ** period - 1) / rate
    if prepaid: coef *= (1 + rate)
    return coef

def afv(annuity, rate, period, prepaid = False):
    """Future value of annuity."""
    return annuity * afv_coef(rate, period, prepaid)

def comp_rate(pval, fval, period):
    """Compound interest/growth rate."""
    return (fval / pval) ** (1 / period) - 1

def spv(payments, rate, prepaid = False):
    """Present value of a series of various payments."""
    pval = 0
    mark = 0
    period = len(payments)
    n = 1
    if prepaid:
        pval = payments[mark]
        mark += 1
    while mark < period:
        pval = pval + pv(payments[mark], rate, n)
        n += 1
        mark += 1
    return pval

def sfv(payments, rate, prepaid = False):
    """Future value of a series of various payments."""
    fval = 0
    period = len(payments)
    mark = period - 1
    n = 1
    if not prepaid:
        fval = payments[mark]
        mark -= 1
    while mark >= 0:
        fval = fval + fv(payments[mark], rate, n)
        n += 1
        mark -= 1
    return fval

def mean(data):
    """Arithmatic mean."""
    return sum(data) / len(data)

def median(data, bSorted = False):
    """Median of a sequence of data."""
    if not bSorted: return median(sorted(data), True)
    n = len(data)
    mark = n // 2
    if n % 2 == 0: return (data[mark-1] + data[mark]) / 2
    else : return data[mark]

def var(data, sample = True):
    """Variance."""
    avg = mean(data)
    dev = 0
    n = len(data)
    for i in range(n): dev += (data[i] - avg) ** 2
    if sample: return dev / (n - 1)
    else : return dev / n

def sd(data, sample = True):
    """Standard deviation."""
    return math.sqrt(var(data, sample))

def cov(data1, data2, sample = True):
    """Covariance."""
    avg1 = mean(data1)
    avg2 = mean(data2)
    n = len(data1)
    dev = 0
    for i in range(n): dev += (data1[i] - avg1) * (data2[i] - avg2)
    if sample: return dev / (n - 1)
    else : return dev / n

def cor(data1, data2):
    """Correlation coefficient."""
    return cov(data1, data2) / (sd(data1) * sd(data2))

def moment(data, k, central = True):
    """Moment."""
    n = len(data)
    result = 0
    if central:
        mu = mean(data)
        for i in range(n): result += (data[i] - mu) ** k
    else:
        for i in range(n): result += data[i] ** k
    return result / n

def skew(data):
    """Skewness."""
    return moment(data, 3) / math.pow(moment(data, 2), 1.5)

def kurt(data, excess = True):
    """Kurtosis."""
    result = moment(data, 4) / math.pow(moment(data, 2), 2)
    if excess: result -= 3
    return result

def dnorm(x, mu = 0, sigma = 1):
    """Probability density of normal distribution."""
    z = (x - mu) / sigma
    return 1 / (math.sqrt(2 * math.pi) * sigma) * math.exp(-0.5 * z * z)

def pnorm(x, mu = 0, sigma = 1, eps = Epsilon()):
    """Cumulative probability (CDF) of normal distribution."""
    z = (x - mu) / sigma
    d = dnorm(z)
    s, sn = z, z
    p = 0.5 + d * s
    i = 3
    while True:
        sn *= z * z / i
        s += sn
        pn = 0.5 + d * s
        if eps(pn - p):
            p = pn
            break
        p = pn
        i += 2
    return p

def qnorm(p, mu = 0, sigma = 1, eps = Epsilon()):
    """Quantile of normal distribution."""
    peps = Epsilon(1e-15)
    if peps(p - 0.5): return mu
    lx, rx = 0, 0
    if p > 0.5:
        rx = sigma
        while pnorm(rx, mu, sigma, peps) < p: rx += sigma
        lx = rx - sigma
    else:
        lx = -sigma
        while pnorm(lx, mu, sigma, peps) > p: lx -= sigma
        rx = lx + sigma
    if peps(p - pnorm(lx, mu, sigma, peps)): return lx
    if peps(p - pnorm(rx, mu, sigma, peps)): return rx
    while True:
        mid = (lx + rx) / 2
        pmid = pnorm(mid, mu, sigma, peps)
        if eps(rx - lx) or peps(p - pmid): return mid
        if pmid > p: rx = mid
        else: lx = mid

def dt(x, n):
    """Probability density of t distribution."""
    return gamma((n + 1) / 2) / (math.sqrt(n * math.pi) * gamma(n / 2)) * math.pow(1 + x * x / n, (n + 1) / -2)

def pt(x, n, eps = Epsilon()):
    """Cumulative probability (CDF) of t distribution."""
    if x == 0: return 0.5
    if (x > 0):
        p = integrate(lambda y: dt(y, n), (-x, x), eps)
        return p + (1 - p) / 2
    else:
        p = integrate(lambda y: dt(y, n), (x, -x), eps)
        return (1 - p) / 2

def qt(p, n, eps = Epsilon()):
    """Quantile of t distribution."""
    peps = Epsilon(1e-10)
    if peps(p - 0.5): return 0
    lx, rx = 0, 0
    if p > 0.5:
        rx = 1
        while pt(rx, n, peps) < p: rx += 1
        lx = rx - 1
    else:
        lx = -1
        while pt(lx, n, peps) > p: lx -= 1
        rx = lx + 1
    if peps(p - pt(lx, n, peps)): return lx
    if peps(p - pt(rx, n, peps)): return rx
    while True:
        mid = (lx + rx) / 2
        pmid = pt(mid, n, peps)
        if eps(rx - lx) or peps(p - pmid): return mid
        if pmid > p: rx = mid
        else: lx = mid

def pois(lmd, k, cum = False):
    """Probability of poisson distribution."""
    if not cum: return math.pow(lmd, k) * math.exp(-lmd) / math.factorial(k)
    else:
        result = 0
        for i in range(k + 1): result += pois(lmd, i, False)
        return result

def binom(n, p, k, cum = False):
    """Probability of binomial distribution."""
    if not cum: return comb(n, k) * math.pow(p, k) * math.pow(1 - p, n - k)
    else:
        result = 0
        for i in range(k + 1): result += binom(n, p, i, False)
        return result

def expo(theta, x):
    """Cumulative probability of exponential distribution."""
    if x > 0: return 1 - math.exp(-1 / theta * x)
    else: return 0

def gamma(x):
    """Gamma function."""
    if not x > 0: raise RuntimeError("Invalid argument.")
    if math.modf(x)[0] == 0: return math.factorial(x - 1)
    p = (1.000000000190015, 76.18009172947146, -86.50532032941677, 24.01409824083091, -1.231739572450155, 1.208650973866179e-3, -5.395239384953e-6)
    s = p[0]
    for i in range(1, 7): s += p[i] / (x + i)
    return math.sqrt(2 * math.pi) / x * s * math.pow(x + 5.5, x + 0.5) * math.exp(-x - 5.5)

def rnorm(n = 1, mu = 0, sigma = 1):
    """Random numbers of normal distribution."""
    if n == 1:
        u = random.uniform(0, 1)
        v = random.uniform(0, 1)
        z = math.sqrt(-2 * math.log(u)) * math.cos(2 * math.pi * v)
        return mu + sigma * z
    else:
        result = []
        for i in range(n): result.append(rnorm(1, mu, sigma))
        return result

def rpois(lmd, n = 1):
    """Random numbers of poisson distribution."""
    if n == 1:
        L = math.exp(-lmd)
        p = 1
        k = 0
        while True:
            k += 1
            p *= random.uniform(0, 1)
            if not p > L: break
        return k - 1
    else:
        result = []
        for i in range(n): result.append(rpois(lmd))
        return result

def rbinom(n, p, count = 1):
    """Random numbers of binomial distribution."""
    if count == 1:
        k = 0
        for i in range(n):
            if randp(p): k += 1
        return k
    else:
        result = []
        for i in range(count): result.append(rbinom(n, p))
        return result

def rexp(theta, n = 1):
    """Random numbers of exponential distribution."""
    if n == 1: return -theta * math.log(random.uniform(0, 1))
    else:
        result = []
        for i in range(n): result.append(rexp(theta))
        return result

def fac(n):
    """Factorial."""
    return math.factorial(n)

def randf(low, high, n = 1):
    """Floating point random numbers in rang [low, high] inclusive."""
    if n == 1: return random.uniform(low, high)
    else:
        result = []
        for i in range(n): result.append(random.uniform(low, high))
        return result

def randi(low, high, n = 1):
    """Integer random numbers in range [low, high] inclusive."""
    if n == 1: return random.randint(low, high)
    else:
        result = []
        for i in range(n): result.append(random.randint(low, high))
        return result

def randp(p):
    """0~1 distribution."""
    return not random.uniform(0, 1) > p

def randseed(s = None):
    """Initialize the random number generator."""
    random.seed(s)

def linregress(xdata, ydata):
    """Linear regression."""
    slope = cov(xdata, ydata) / var(xdata)
    intercept = mean(ydata) - slope * mean(xdata)
    return (slope, intercept)

def seq(frm, to, step = 1):
    """Generate a list from a sequence of numbers."""
    result = []
    if step > 0:
        if frm > to: return result
        while True:
            result.append(frm)
            frm += step
            if frm > to: break
    elif step < 0:
        if frm < to: return result
        while True:
            result.append(frm)
            frm += step
            if frm < to: break
    return result

def linspace(start, stop, count = 100):
    """Generate a list from a sequence of numbers uniformly spread a range."""
    result = []
    if count < 2: return result
    elif count == 2:
        result.append(start)
        result.append(stop)
    else:
        delta = (stop - start) / (count - 1)
        result.append(start)
        for i in range(count - 2):
            start += delta
            result.append(start)
        result.append(stop)
    return result

def add(*vects):
    """Vector addition."""
    result = []
    n = len(vects)
    l = len(vects[0])
    for i in range(l):
        val = 0
        for j in range(n): val += vects[j][i]
        result.append(val)
    return result

def sadd(vect, scalar):
    """Elementwise scalar addition."""
    return [x + scalar for x in vect]

def sub(*vects):
    """Vector subtraction."""
    result = []
    n = len(vects)
    l = len(vects[0])
    for i in range(l):
        val = vects[0][i]
        for j in range(1, n): val -= vects[j][i]
        result.append(val)
    return result

def ssub(vect, scalar, direction = True):
    """Elementwise scalar subtraction."""
    if direction: return [x - scalar for x in vect]
    else: return [scalar - x for x in vect]

def mul(*vects):
    """Elementwise vector multiplication."""
    result = []
    n = len(vects)
    l = len(vects[0])
    for i in range(l):
        val = 1
        for j in range(n): val *= vects[j][i]
        result.append(val)
    return result

def smul(vect, scalar):
    """Elementwise scalar multiplication."""
    return [x * scalar for x in vect]

def div(*vects):
    """Elementwise vector division."""
    result = []
    n = len(vects)
    l = len(vects[0])
    for i in range(l):
        val = vects[0][i]
        for j in range(1, n): val /= vects[j][i]
        result.append(val)
    return result

def sdiv(vect, scalar, direction = True):
    """Elementwise scalar division."""
    if direction: return [x / scalar for x in vect]
    else: return [scalar / x for x in vect]

def dot_prod(vect1, vect2):
    """Dot product of two vectors."""
    result = 0
    for i in range(len(vect1)): result += vect1[i] * vect2[i]
    return result

def cross_prod(vect1, vect2):
    """Cross product of two 3-dimensional vectors."""
    result = []
    result.append(vect1[1] * vect2[2] - vect1[2] * vect2[1])
    result.append(vect1[2] * vect2[0] - vect1[0] * vect2[2])
    result.append(vect1[0] * vect2[1] - vect1[1] * vect2[0])
    return result

def norm(vect):
    """Euclidean norm (length) of a vector."""
    return math.sqrt(dot_prod(vect, vect))

def prec(vect, ndec = 4):
    """Round every component of a vector to specific decimal digits."""
    result = []
    n = len(vect)
    for i in range(n): result.append(round(vect[i], ndec))
    return result

def load(fobj, delim = " "):
    """Load data from file."""
    result = []
    line = fobj.readline()
    if line != "":
        items = line.strip().split(delim)
        for item in items: result.append(float(item.strip()))
    return result

def save(fobj, data, delim = " "):
    """Save data to file."""
    n = len(data)
    for i in range(n):
        fobj.write(str(data[i]))
        if i != n - 1: fobj.write(delim)
    fobj.write("\n")
    fobj.flush()

def load_table(fobj, nrow, ncol, delim = " "):
    """Load table/matrix data from file."""
    result = []
    for i in range(ncol): result.append([])
    for j in range(nrow):
        row = load(fobj, delim)
        for k in range(ncol): result[k].append(row[k])
    return result

def save_table(fobj, data, delim = " "):
    """Save table/matrix data to file."""
    nrow = len(data[0])
    ncol = len(data)
    for i in range(nrow):
        row = []
        for j in range(ncol): row.append(data[j][i])
        save(fobj, row, delim)

def perm(m, n):
    """Permutation number (mPn)."""
    result = 1
    while n > 0:
        result *= m
        m -= 1
        n -=1
    return result

def comb(m, n):
    """Combination number (mCn)."""
    if m - n < n: return comb(m, m - n)
    return perm(m, n) // math.factorial(n)

def gcd(a, b):
    """Greatest common divisor."""
    while b != 0:
        temp = b
        b = a % b
        a = temp
    return a

def lcm(a, b):
    """Least common multiple."""
    return a * b // gcd(a, b)

def probe(func, guess, factor = 1.6, ntry = 50):
    """Find a region contains at least one root."""
    lv = func(guess[0])
    rv = func(guess[1])
    for i in range(ntry):
        if lv * rv <= 0: return True
        if abs(lv) < abs(rv):
            guess[0] -= factor * (guess[1] - guess[0])
            lv = func(guess[0])
        else:
            guess[1] += factor * (guess[1] - guess[0])
            rv = func(guess[1])
    return False

def scan(func, scope, nr):
    """Scan a specific scope to find sub-regions contain root."""
    result = []
    marks = linspace(scope[0], scope[1], nr + 1)
    for i in range(nr):
        if func(marks[i]) * func(marks[i+1]) <= 0:
            result.append((marks[i], marks[i+1]))
    return result

def solve(func, region, eps = Epsilon()):
    """Find a root in a specific region using binary search algorithm."""
    lv = region[0]
    rv = region[1]
    if func(lv) == 0: return lv
    if func(rv) == 0: return rv
    while True:
        mid = (lv + rv) / 2
        if eps(rv - lv) or func(mid) == 0: return mid
        if func(lv) * func(mid) < 0: rv = mid
        else : lv = mid

def integrate(func, region, eps = Epsilon()):
    """Numerical integration."""
    n = int((region[1] - region[0]) * 2)
    if n < 2: n = 2
    marks = linspace(region[0], region[1], n + 1)
    s1 = func(marks[0]) + func(marks[n])
    s2 = 0
    for i in range(1, n): s2 += func(marks[i])
    s2 *= 2
    s4 = 0
    for i in range(n): s4 += func((marks[i] + marks[i+1]) / 2)
    s4 *= 4
    s = (region[1] - region[0]) / n / 6 * (s1 + s2 + s4)
    k = 2
    while True:
        n *= k
        marks = linspace(region[0], region[1], n + 1)
        s1 = func(marks[0]) + func(marks[n])
        s2 = 0
        for i in range(1, n): s2 += func(marks[i])
        s2 *= 2
        s4 = 0
        for i in range(n): s4 += func((marks[i] + marks[i+1]) / 2)
        s4 *= 4
        sn = (region[1] - region[0]) / n / 6 * (s1 + s2 + s4)
        if eps(sn - s):
            s = sn
            break
        s = sn
        k += 1
    return s

class Matrix:
    """Matrix class. The row and column indices start from zero."""
    def __init__(self, data = []):
        """Construct a matrix."""
        if len(data) != 0:
            self.dimen = (len(data[0]), len(data))
            self.data = copy.deepcopy(data)
        else:
            self.dimen = (0, 0)
            self.data = []
    
    def getData(self):
        """Get matrix data."""
        return copy.deepcopy(self.data)

    def setData(self, data):
        """Set matrix data."""
        if len(data) != 0:
            self.dimen = (len(data[0]), len(data))
            self.data = copy.deepcopy(data)
        else:
            self.dimen = None
            self.data = None

    def dim(self):
        """Get the dimension of the matrix."""
        return self.dimen

    def nrow(self):
        """Get the number of rows."""
        return self.dimen[0]

    def ncol(self):
        """Get the number of columns."""
        return self.dimen[1]

    def square(self):
        """Test if this matrix is a square matrix."""
        return self.dimen[0] == self.dimen[1]

    def symmetric(self):
        """Test if this matrix is symmetric."""
        if not self.square(): return False
        return self == self.transpose()

    def get(self, m, n):
        """Get a specific element of the matrix."""
        return self.data[n][m]

    def set(self, m, n, val):
        """Set a specific element of the matrix."""
        self.data[n][m] = val

    def getRow(self, m):
        """Get a row of the matrix."""
        result = []
        for i in range(self.dimen[1]): result.append(self.data[i][m])
        return result

    def getCol(self, n):
        """Get a column of the matrix."""
        return copy.deepcopy(self.data[n])

    def setRow(self, m, vect):
        """Set a row of the matrix."""
        for i in range(self.dimen[1]): self.data[i][m] = vect[i]

    def setCol(self, n, vect):
        """Set a col of the matrix."""
        self.data[n] = copy.deepcopy(vect)

    def exchange(self, i, j, row = True):
        """Exchange two rows or two columns."""
        if row:
            temp = self.getRow(i)
            self.setRow(i, self.getRow(j))
            self.setRow(j, temp)
        else:
            temp = self.getCol(i)
            self.setCol(i, self.getCol(j))
            self.setCol(j, temp)

    def transpose(self):
        """Transpose matrix."""
        tbl = []
        for i in range(self.dimen[0]): tbl.append(self.getRow(i))
        return Matrix(tbl)

    def rank(self, eps = Epsilon()):
        """Rank of matrix."""
        mat = Matrix(self.data)
        m, n = mat.dimen
        limit = n
        if m < n: limit = m
        r = 0
        for i in range(n):
            if eps(mat(r, i)):
                exchanged = False
                for j in range(r + 1, m):
                    if not eps(mat(j, i)):
                        mat.exchange(r, j)
                        exchanged = True
                        break
                if not exchanged: continue
            for k in range(r + 1, m):
                if not eps(mat(k, i)):
                    s = mat(k, i) / mat(r, i)
                    mat.setRow(k, add(smul(mat.getRow(r), -s), mat.getRow(k)))
            r += 1
            if r == limit: break
        return r

    def submat(self, frow, lrow, fcol, lcol):
        """Submatrix."""
        tbl = []
        m, n = lrow - frow + 1, lcol - fcol + 1
        for i in range(n): tbl.append([0] * m)
        for i in range(m):
            for j in range(n):
                tbl[j][i] = self.data[fcol + j][frow + i]
        return Matrix(tbl)
    
    def resid(self, i, j):
        """Residual matrix with the ith row and jth column excluded."""
        tbl = []
        for n in range(self.dimen[1]):
            if n != j:
                col = self.getCol(n)
                del col[i]
                tbl.append(col)
        return Matrix(tbl)

    def __eq__(self, mat):
        """Matrix equality."""
        return self.dimen == mat.dimen and self.data == mat.data

    def __ne__(self, mat):
        """Matrix inequality."""
        return not self.__eq__(mat)

    def __getitem__(self, n):
        """Get a column of the matrix."""
        return self.data[n]

    def __setitem__(self, n, vect):
        """Set a column of the matrix."""
        self.data[n] = vect

    def __call__(self, m, n):
        """Access a specific element of the matrix."""
        return self.data[n][m]

    def __repr__(self):
        """The string representation of a matrix."""
        return repr(self.transpose().getData())

    def __add__(self, mat):
        """Matrix addition."""
        if not isinstance(mat, Matrix): return NotImplemented
        if self.dimen != mat.dimen: raise RuntimeError("Incompatible matrices.")
        tbl = []
        for i in range(self.dimen[1]): tbl.append(add(self.data[i], mat.data[i]))
        return Matrix(tbl)

    def __sub__(self, mat):
        """Matrix subtraction."""
        if not isinstance(mat, Matrix): return NotImplemented
        if self.dimen != mat.dimen: raise RuntimeError("Incompatible matrices.")
        tbl = []
        for i in range(self.dimen[1]): tbl.append(sub(self.data[i], mat.data[i]))
        return Matrix(tbl)

    def __mul__(self, obj):
        """Matrix multiplication and Scalar multiplication."""
        if isinstance(obj, Matrix):
            if self.dimen[1] != obj.dimen[0]: raise RuntimeError("Incompatible matrices.")
            m, n = self.dimen[0], obj.dimen[1]
            tbl = table([0] * m * n, m, n)
            for i in range(n):
                for j in range(self.dimen[1]):
                    tbl[i] = add(tbl[i], smul(self.data[j], obj.data[i][j]))
            return Matrix(tbl)
        elif isinstance(obj, numbers.Number):
            tbl = []
            n = self.dimen[1]
            for i in range(n): tbl.append(smul(self.data[i], obj))
            return Matrix(tbl)
        else: return NotImplemented

    def __rmul__(self, scalar):
        """Leftside scalar multiplication."""
        if not isinstance(scalar, numbers.Number): return NotImplemented
        return self.__mul__(scalar)

    def __round__(self, ndec = 4):
        """Round to specific decimal digits."""
        mat = Matrix(self.data)
        m, n = mat.dim()
        for i in range(m):
            for j in range(n):
                mat.set(i, j, round(mat(i, j), ndec))
        return mat

def matrix(m, n, val = 0):
    """Generate a matrix with specific value."""
    return Matrix(table([val] * m * n, m, n))

def v2mat(vect, m, n, byrow = True):
    """Generate a matrix from a vector."""
    return Matrix(table(vect, m, n, byrow))

def rowmat(vect):
    """Generate an one row matrix (row vector in linear algebra)."""
    return v2mat(vect, 1, len(vect))

def colmat(vect):
    """Generate an one column matrix (column vector in linear algebra)."""
    return v2mat(vect, len(vect), 1)

def cbind(*arg):
    """Construct a matrix by column binding."""
    s = len(arg)
    m = 0
    tbl = []
    for i in range(s):
        if isinstance(arg[i], Matrix):
            if i == 0: m = arg[i].nrow()
            if arg[i].nrow() != m: raise RuntimeError("Incompatible argument.")
            n = arg[i].ncol()
            for j in range(n): tbl.append(arg[i].getCol(j))
        elif isinstance(arg[i], list):
            if i == 0: m = len(arg[i])
            if len(arg[i]) != m: raise RuntimeError("Incompatible argument.")
            tbl.append(arg[i])
        else: raise RuntimeError("Invalid argument.")
    return Matrix(tbl)

def rbind(*arg):
    """Construct a matrix by row binding."""
    s = len(arg)
    n = 0
    tbl = []
    for i in range(s):
        if isinstance(arg[i], Matrix):
            if i == 0:
                n = arg[i].ncol()
                for j in range(n): tbl.append([])
            if arg[i].ncol() != n: raise RuntimeError("Incompatible argument.")
            m = arg[i].nrow()
            for j in range(m):
                row = arg[i].getRow(j)
                for k in range(n): tbl[k].append(row[k])
        elif isinstance(arg[i], list):
            if i == 0:
                n = len(arg[i])
                for j in range(n): tbl.append([])
            if len(arg[i]) != n: raise RuntimeError("Incompatible argument.")
            for j in range(n): tbl[j].append(arg[i][j])
        else: raise RuntimeError("Invalid argument.")
    return Matrix(tbl)

def t(mat):
    """Transpose matrix."""
    return mat.transpose()

def eye(n):
    """Generate an identity matrix."""
    mat = matrix(n, n)
    for i in range(n): mat.set(i, i, 1)
    return mat

def diag(obj):
    """Diagonal elements and diagonal matrix."""
    if isinstance(obj, list):
        n = len(obj)
        mat = matrix(n, n)
        for i in range(n): mat.set(i, i, obj[i])
        return mat
    elif isinstance(obj, Matrix):
        n = obj.ncol()
        result = []
        for i in range(n): result.append(obj(i, i))
        return result
    else: raise RuntimeError("Need a vector or a matrix argument.")

def det(mat, eps = Epsilon()):
    """Determinant."""
    if not mat.square(): raise RuntimeError("Square matrix needed.")
    n = mat.ncol()
    if n == 1:
        if eps(mat(0, 0)): return 0
        else: return mat(0, 0)
    result = 0
    for i in range(n):
        if not eps(mat(0, i)):
            resid = mat(0, i) * det(mat.resid(0, i))
            if i % 2 != 0: resid = -resid
            result += resid
    return result

def inv(mat, eps = Epsilon()):
    """Inverse matrix."""
    if not mat.square(): raise RuntimeError("Square matrix needed.")
    n = mat.ncol()
    I = eye(n)
    tbl = []
    lup = lu(mat, eps)
    for i in range(n):
        result = lusolve(lup, I.getCol(i), eps)
        if result == (): raise RuntimeError("Singular matrix.")
        tbl.append(list(result))
    return Matrix(tbl)

def trace(mat, eps = Epsilon()):
    """Trace (sum of main diagonal elements) of square matrix."""
    if not mat.square(): raise RuntimeError("Square matrix needed.")
    n = mat.ncol()
    val = 0
    for i in range(n):
        if not eps(mat(i, i)): val += mat(i, i)
    return val

def positive(mat, eps = Epsilon()):
    """Check whether a symmetric matrix is positive definite."""
    if not mat.symmetric(): raise RuntimeError("Symmetric matrix needed.")
    lup = lu(mat, eps)
    if lup == (): return False
    U = lup[1]
    n = U.ncol()
    for i in range(n):
        if U(i, i) < 0: return False
    return True

def lu(mat, eps = Epsilon()):
    """LU decomposition."""
    if not mat.square(): raise RuntimeError("Square matrix needed.")
    n = mat.ncol()
    L = eye(n)
    P = eye(n)
    U = Matrix(mat.data)
    for i in range(n):
        if eps(U(i, i)):
            exchanged = False
            for k in range(i + 1, n):
                if not eps(U(k, i)):
                    U.exchange(i, k)
                    P.exchange(i, k)
                    for j in range(i):
                        temp = L(i, j)
                        L.set(i, j, L(k, j))
                        L.set(k, j, temp)
                    exchanged = True
                    break
            if not exchanged: return ()
        for j in range(i + 1, n):
            if not eps(U(j, i)):
                s = U(j, i) / U(i, i)
                L.set(j, i, s)
                U.setRow(j, add(smul(U.getRow(i), -s), U.getRow(j)))
    return (L, U, P)

def qr(mat, eps = Epsilon()):
    """QR decomposition."""
    n = mat.ncol()
    if mat.rank(eps) < n: return ()
    tbl = []
    for i in range(n):
        vec = mat.getCol(i)
        for j in range(i): vec = sub(vec, smul(tbl[j], dot_prod(tbl[j], vec)))
        tbl.append(sdiv(vec, norm(vec)))
    Q = Matrix(tbl)
    R = matrix(n, n)
    for i in range(n):
        for j in range(i, n):
            R.set(i, j, dot_prod(tbl[i], mat.getCol(j)))
    return (Q, R)

def linsolve(mat, vec, eps = Epsilon()):
    """Solve a linear system (Ax = b)."""
    return lusolve(lu(mat, eps), vec, eps)

def lusolve(lup, vec, eps = Epsilon()):
    """Solve a linear system represented by L, U, P (Ax = b, PA = LU)."""
    if lup == (): return ()
    L, U, P = lup
    n = U.nrow()
    if n != len(vec): raise RuntimeError("Incompatible arguments.")
    b = colmat(vec)
    if P != eye(n): b = P * b
    result = []
    for i in range(n):
        val = 0
        for j in range(i):
            if not eps(L(i, j)): val += L(i, j) * result[j]
        result.append(b(i, 0) - val)
    b = colmat(result)
    for i in range(n - 1, -1, -1):
        val = 0
        for j in range(n - 1, i, -1):
            if not eps(U(i, j)): val += U(i, j) * result[j]
        result[i] = (b(i, 0) - val) / U(i, i)
    return tuple(result)

def qrsolve(qrmat, vec, eps = Epsilon()):
    """Solve a linear system represented by Q, R (Ax = b, A = QR)."""
    if qrmat == (): return ()
    Q, R = qrmat
    n = R.nrow()
    if Q.nrow() != len(vec): raise RuntimeError("Incompatible arguments.")
    b = colmat(vec)
    b = t(Q) * b
    result = [0] * n
    for i in range(n - 1, -1, -1):
        val = 0
        for j in range(n - 1, i, -1):
            if not eps(R(i, j)): val += R(i, j) * result[j]
        result[i] = (b(i, 0) - val) / R(i, i)
    return tuple(result)

def table(vect, m, n, byrow = True):
    """Generate a table from a vector."""
    tbl = []
    for i in range(n): tbl.append([])
    if byrow:
        k = 0
        for i in range(m):
            for j in range(n):
                tbl[j].append(vect[k])
                k += 1
    else:
        k = 0
        for i in range(n):
            for j in range(m):
                tbl[i].append(vect[k])
                k += 1
    return tbl

def to_str(mat, bycol = True):
    """Generate string representation of a matrix."""
    result = ""
    if bycol:
        n = mat.ncol()
        for i in range(n):
            result += str(mat.getCol(i))
            if i != n - 1: result += ", "
        return "[" + result + "]"
    else: return str(mat)
